function [accuracy confusion detailedAccuracies] = EvaluateClassifications(clips,contextDimension,labelStrings, verbose)

if (not(exist('verbose')))
    verbose = 0;
end
classification = [];   % Format of classification is   [TS p1 p2 .. pn decision label] one row per slice

if (strcmp(contextDimension,'posture')==1),
    for i=1:length(clips)
        classification = [classification; clips{i}.classificationPosture];
    end
end

if (strcmp(contextDimension,'event')==1),
    for i=1:length(clips)
        classification = [classification; clips{i}.classificationEvent];
    end
end

if (strcmp(contextDimension,'speech')==1),
    for i=1:length(clips)
        classification = [classification; clips{i}.classificationSpeech]; 
    end
end

if (strcmp(contextDimension,'location')==1),
    for i=1:length(clips)
        classification = [classification; clips{i}.classificationLocation];
    end
end

NUM_CLASSES = length(labelStrings);

if isempty(classification),
    fprintf('EvaluateClassifications: Error, unknown context dimension ''%s''\n',contextDimension);
    return;
end

if (verbose)
    set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
    subplot(2,1,1);
    hold on;
    plot(classification(:,end-1)*10,'b--');  % decision
    plot(classification(:,end)*10,'k--');  % label
    title(sprintf('Test data with esimates (blue dashed) and ground truth (black dashed) for ''%s''',contextDimension));
    subplot(2,1,2);
    hold on;
    plot(classification(:,2:end-2)); % probabilities
    correctClassifications = find(classification(:,end-1)==classification(:,end));
    wrongClassifications = find(classification(:,end-1)~=classification(:,end));
    plot(correctClassifications,ones(1,length(correctClassifications))*0.5,'g.');
    plot(wrongClassifications,ones(1,length(wrongClassifications))*0.4,'r.');
    title('Probability of the different classes. Green and red are correct and wrong classifications.');
    drawnow;
end
%% Now calculate the accuracy: correct/total * 100%
accuracy=100*sum(classification(:,end-1)==classification(:,end))/length(classification);
fprintf('Acc for ''%s'' is %3.1f%% (',contextDimension,accuracy);
drawnow;

% compute confusion matrix
confusion = zeros(NUM_CLASSES, NUM_CLASSES);
for k=1: NUM_CLASSES
    for j=1: NUM_CLASSES
     index = find(classification(:,end) == k);
      confusion(k,j) = length(find(classification(index,end-1) == j));
    end
end
%confusion = confusion ./ length(test);

detailedAccuracies = 100*(diag(confusion)./sum(confusion,2))';
fprintf('%2.0f%% ',detailedAccuracies);
fprintf('avg: %2.0f%%)\n', mean(detailedAccuracies));
confusion