function [eval] = EvaluatePostClassifications(postClass,labels,n_states,chainStrings,verbose,preClass,stateStrings)
%
% [accuracies confusions detailedAccuracies] =
% EvaluatePostClassifications(postClass,labels,n_states,chainStrings,verbose,preClass)
%

%% Spacing optimal for import in Excel

if (exist('preClass'))
    numCases = size(labels,1);
    totalPreClassificationErros = 0;
    totalPostClassificationErros = 0;
    totalImprovements = 0;
    totalWorsening = 0;
    totalUnimproved = 0;
    for X=1:length(n_states)
        preClassificationErros{X} = find(preClass(:,X)~=labels(:,X));
        postClassificationErros{X} = find(postClass(:,X)~=labels(:,X));
        improvements{X} = find(preClass(:,X)~=labels(:,X) & postClass(:,X)==labels(:,X) );
        worsening{X}    = find(preClass(:,X)==labels(:,X) & postClass(:,X)~=labels(:,X));
        unimproved{X}   = find(preClass(:,X)~=labels(:,X) & postClass(:,X)~=labels(:,X));
        totalPreClassificationErros = totalPreClassificationErros + length(preClassificationErros{X});
        totalPostClassificationErros = totalPostClassificationErros + length(postClassificationErros{X});
        totalImprovements = totalImprovements + length(improvements{X});
        totalWorsening = totalWorsening + length(worsening{X});
        totalUnimproved = totalUnimproved + length(unimproved{X});
    end
    fprintf('\tpre-classification errors\tpost-classification errors\timproved\tworse\tunimproved\n');
    for X=1:length(n_states)
        fprintf('%s:\t',chainStrings{X});
        fprintf('%6d (%2.0f%%)\t',length(preClassificationErros{X}),100/numCases*length(preClassificationErros{X}));
        fprintf('%6d (%2.0f%%)\t',length(postClassificationErros{X}),100/numCases*length(postClassificationErros{X}));
        fprintf('%6d\t',length(improvements{X}));
        fprintf('%6d\t',length(worsening{X}));
        fprintf('%6d\n',length(unimproved{X}));
    end
    fprintf('Total:\t');
    fprintf('%6d (%2.0f%%)\t',totalPreClassificationErros,100/(numCases*4)*totalPreClassificationErros);
    fprintf('%6d (%2.0f%%)\t',totalPostClassificationErros,100/(numCases*4)*totalPostClassificationErros);
    fprintf('%6d\t',totalImprovements);
    fprintf('%6d\t',totalWorsening);
    fprintf('%6d\n',totalUnimproved);
    if (verbose)
        set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
        X = verbose;
        plot(1:numCases,labels(:,X),'x', improvements{X},preClass(improvements{X},X),'ro',...
            worsening{X},postClass(worsening{X},X),'ks', unimproved{X},preClass(unimproved{X},X),'md',...
            1:numCases,preClass(:,1)-0.07,'c-.', 1:numCases,preClass(:,2)-0.03,'g-.',...
            1:numCases,preClass(:,3)+0.03,'m-.', 1:numCases,preClass(:,4)+0.07,'y-.');
        legend('correct evidence','improved by filtering',...
            'extra errors','unimproved by filtering',chainStrings{:});
        set(gca,'ytick',1:n_states(X));
        set(gca,'yticklabel',stateStrings{X});
        title(chainStrings{X});
    end
    eval.totalPreClassificationErros = totalPreClassificationErros;
    eval.totalPostClassificationErros = totalPostClassificationErros;
    eval.preClassificationErros = preClassificationErros;
    eval.postClassificationErros = postClassificationErros;
    eval.improvements = improvements;
    eval.worsening = worsening;
    eval.unimproved = unimproved;
end
fprintf('\toverall accuracy\tclass average\n');
for i=1:length(n_states)

    %% Calculate the accuracy: correct/total * 100%
    accuracies{i}=100*sum(postClass(:,i)==labels(:,i))/length(labels);
    
    fprintf('%s:\t%3.1f%%\t',chainStrings{i},accuracies{i} );

    % Compute confusion matrix
    NUM_CLASSES = n_states(i);
    confusions{i} = zeros(NUM_CLASSES, NUM_CLASSES);
    for k=1: NUM_CLASSES
        for j=1: NUM_CLASSES
            index = find(labels(:,i) == k);
            confusions{i}(k,j) = length(find(postClass(index,i) == j));
        end
    end
    %confusions{i} = confusions{i} ./ length(labels);
    detailedAccuracies{i} = 100*(diag(confusions{i})./sum(confusions{i},2))';
    classAverage = mean(detailedAccuracies{i}(find(~isnan(detailedAccuracies{i}))));
    fprintf('%2.0f%%\t\t',classAverage);
    fprintf('%2.0f%%\t',detailedAccuracies{i});
    fprintf('\n');
    
    if (0)
        confusions{i}

        set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
        plot(postClass(:,i),'b--');  % decision
        hold on;
        plot(labels(:,i),'k--');  % label
        title(sprintf('Test data with esimates (blue dashed) and ground truth (black dashed) for ''%s''',chainStrings{i}));
    end
end


%% Spacing optimal for MATLAB view

% if (exist('preClass'))
%     numCases = size(labels,1);
%     totalPreClassificationErros = 0;
%     totalPostClassificationErros = 0;
%     for X=1:length(n_states)
%         preClassificationErros{X} = find(preClass(:,X)~=labels(:,X));
%         postClassificationErros{X} = find(postClass(:,X)~=labels(:,X));
%         improvements{X} = find(preClass(:,X)~=labels(:,X) & postClass(:,X)==labels(:,X) );
%         worsening{X}    = find(preClass(:,X)==labels(:,X) & postClass(:,X)~=labels(:,X));
%         unimproved{X}   = find(preClass(:,X)~=labels(:,X) & postClass(:,X)~=labels(:,X));
%         totalPreClassificationErros = totalPreClassificationErros + length(preClassificationErros{X});
%         totalPostClassificationErros = totalPostClassificationErros + length(postClassificationErros{X});
%     end
%     fprintf('\t\t\tpreClassErr\t\tpostClassErr\timproved\t worse\tunimproved\n');
%     for X=1:length(n_states)
%         fprintf('%s:\t',chainStrings{X});
%         fprintf('%6d (%2.0f%%)\t',length(preClassificationErros{X}),100/numCases*length(preClassificationErros{X}));
%         fprintf('%6d (%2.0f%%)\t',length(postClassificationErros{X}),100/numCases*length(postClassificationErros{X}));
%         fprintf('%6d\t\t',length(improvements{X}));
%         fprintf('%6d\t\t',length(worsening{X}));
%         fprintf('%6d\n',length(unimproved{X}));
%     end
%     fprintf('Total:\t\t');
%     fprintf('%6d (%2.0f%%)\t',totalPreClassificationErros,100/(numCases*4)*totalPreClassificationErros);
%     fprintf('%6d (%2.0f%%)\n',totalPostClassificationErros,100/(numCases*4)*totalPostClassificationErros);
%     if (verbose)
%         set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
%         X = verbose;
%         plot(1:numCases,labels(:,X),'x', improvements{X},preClass(improvements{X},X),'ro',...
%             worsening{X},postClass(worsening{X},X),'ks', unimproved{X},preClass(unimproved{X},X),'md',...
%             1:numCases,preClass(:,1)-0.07,'c-.', 1:numCases,preClass(:,2)-0.03,'g-.',...
%             1:numCases,preClass(:,3)+0.03,'m-.', 1:numCases,preClass(:,4)+0.07,'y-.');
%         legend('correct evidence','improved by filtering',...
%             'extra errors','unimproved by filtering',chainStrings{:});
%         set(gca,'ytick',1:n_states(X));
%         set(gca,'yticklabel',stateStrings{X});
%         title(chainStrings{X});
%     end
%     eval.totalPreClassificationErros = totalPreClassificationErros;
%     eval.totalPostClassificationErros = totalPostClassificationErros;
%     eval.preClassificationErros = preClassificationErros;
%     eval.postClassificationErros = postClassificationErros;
%     eval.improvements = improvements;
%     eval.worsening = worsening;
%     eval.unimproved = unimproved;
% end
% for i=1:length(n_states)
% 
%     %% Calculate the accuracy: correct/total * 100%
%     accuracies{i}=100*sum(postClass(:,i)==labels(:,i))/length(labels);
%     fprintf('Acc for ''%s''\tis %3.1f%% (',chainStrings{i},accuracies{i} );
% 
%     % Compute confusion matrix
%     NUM_CLASSES = n_states(i);
%     confusions{i} = zeros(NUM_CLASSES, NUM_CLASSES);
%     for k=1: NUM_CLASSES
%         for j=1: NUM_CLASSES
%             index = find(labels(:,i) == k);
%             confusions{i}(k,j) = length(find(postClass(index,i) == j));
%         end
%     end
%     %confusions{i} = confusions{i} ./ length(labels);
%     detailedAccuracies{i} = 100*(diag(confusions{i})./sum(confusions{i},2))';
%     fprintf('%2.0f%% ',detailedAccuracies{i});
%     fprintf(')\n');
%     
%     if (0)
%         confusions{i}
% 
%         set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
%         plot(postClass(:,i),'b--');  % decision
%         hold on;
%         plot(labels(:,i),'k--');  % label
%         title(sprintf('Test data with esimates (blue dashed) and ground truth (black dashed) for ''%s''',chainStrings{i}));
%     end
% end

eval.accuracies = accuracies;
eval.detailedAccuracies = detailedAccuracies;
eval.confusions = confusions;