function XML_export_GMM(fileName,bnet,labelStrings)
%
% XML_export_GMM(fileName,bnet,labelStrings)
%
% This function creates an XML representation of a Mixture Model
% with a simple gaussian distributions for each class.
% The output document can read by the C-implemtation of GClassifier
%

fprintf('Exporting GMM Model...');

%%%%% extract values from CPD
mu = get_field(bnet.CPD{2},'mean');
cov = get_field(bnet.CPD{2},'cov');
priors = get_field(bnet.CPD{1},'cpt');
[NUM_FEATURES NUM_CLASSES] = size(mu);

docNode = com.mathworks.xml.XMLUtils.createDocument('MixtureModel');
docRootNode = docNode.getDocumentElement;

docNode.appendChild(docNode.createComment('This file contains the XML representation of a Mixture Model with a simple gaussian distributions for each class. The document can read by the C-implemtation of GClassifier. (c) 2005 MIT Media Lab'));

thisElement = docNode.createElement('Version');
thisElement.appendChild(docNode.createTextNode(sprintf('1.1')));
docRootNode.appendChild(thisElement);

thisElement = docNode.createElement('Dimension');
thisElement.appendChild(docNode.createTextNode(sprintf('%d',NUM_FEATURES)));
docRootNode.appendChild(thisElement);

thisElement = docNode.createElement('Scalar');
thisElement.appendChild(docNode.createTextNode(sprintf('1.0000')));
docRootNode.appendChild(thisElement);

thisElement = docNode.createElement('NumClasses');
thisElement.appendChild(docNode.createTextNode(sprintf('%d',NUM_CLASSES)));
docRootNode.appendChild(thisElement);

thisElement = docNode.createElement('Priors');
thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',priors)));
%thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',ones(1,NUM_CLASSES)/NUM_CLASSES)));  %equal priors
docRootNode.appendChild(thisElement);

for i=1:NUM_CLASSES

    classElement = docNode.createElement('Gaussian');
    docRootNode.appendChild(classElement);
    
    thisElement = docNode.createElement('Label');
    thisElement.appendChild(docNode.createTextNode(sprintf('%s',labelStrings{i})));
    classElement.appendChild(thisElement);

    thisElement = docNode.createElement('Det');
    thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g',det(cov(:,:,i)))));
    classElement.appendChild(thisElement);

    thisElement = docNode.createElement('Mean');
    thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',mu(:,i))));
    classElement.appendChild(thisElement);

    matrixElement = docNode.createElement('InverseCovariance');
    classElement.appendChild(matrixElement);
    
    thisElement = docNode.createElement('NumRows');
    thisElement.appendChild(docNode.createTextNode(sprintf('%d',NUM_FEATURES)));
    matrixElement.appendChild(thisElement);
    thisElement = docNode.createElement('NumCols');
    thisElement.appendChild(docNode.createTextNode(sprintf('%d',NUM_FEATURES)));
    matrixElement.appendChild(thisElement);
    inv_cov = cov(:,:,i)^(-1);
    for j=1:NUM_FEATURES
        thisElement = docNode.createElement('Row');
        thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',inv_cov(j,:))));
        matrixElement.appendChild(thisElement);

    end
end

%%%%% Save the XML document.
xmlwrite(fileName,docNode);
edit(fileName);

fprintf('done.\n');
