function XML_export_IM(fileName,bnet,chainStrings,stateStrings,observationUNLs,outputUNLs)
%
% XML_export_HMM(fileName,bnets,labelStrings)
%
% This function creates an XML representation of a Mixture Model
% with a simple gaussian distributions for each class.
% The output document can read by the C-implemtation of GClassifier
%
% bnets is an array of bnets representing the HMMs (in the format
% specified by Wen Dong, compatible with the influence model)
% labelStrings is a cell-array strings, one for each class

fprintf('Exporting Influence Model...');

%%%%% extract values from CPD

docNode = com.mathworks.xml.XMLUtils.createDocument('InfluenceModel');
docRootNode = docNode.getDocumentElement;

docRootNode.appendChild(docNode.createComment('This file contains the XML representation of an Influence Model. The document can read by the C-implemtation of IMClassifier. (c) 2005 MIT Media Lab'));

thisElement = docNode.createElement('Version');
thisElement.appendChild(docNode.createTextNode(sprintf('1.0')));
docRootNode.appendChild(thisElement);

thisElement = docNode.createElement('NumChains');
thisElement.appendChild(docNode.createTextNode(sprintf('%d',bnet.nchains)));
docRootNode.appendChild(thisElement);

for i=1:bnet.nchains

    chainElement = docNode.createElement('Chain');
    docRootNode.appendChild(chainElement);

    thisElement = docNode.createElement('Name');
    thisElement.appendChild(docNode.createTextNode(sprintf('%s',chainStrings{i})));
    chainElement.appendChild(thisElement);

    thisElement = docNode.createElement('NumStates');
    thisElement.appendChild(docNode.createTextNode(sprintf('%d',bnet.n_states(i))));
    chainElement.appendChild(thisElement);
    
    thisElement = docNode.createElement('DimObservations');
    thisElement.appendChild(docNode.createTextNode(sprintf('%d',bnet.m_outputs(i))));
    chainElement.appendChild(thisElement);
    
    if (bnet.o_type(i) == 'c')
        o_type = 'continuous';
    elseif (bnet.o_type(i) == 'd')
        o_type = 'discrete';
    else
        error('Unknown observation type encountered!');
    end
    thisElement = docNode.createElement('ObservationType');
    thisElement.appendChild(docNode.createTextNode(sprintf('%s',o_type)));
    chainElement.appendChild(thisElement);

    thisElement = docNode.createElement('Priors');
    thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',bnet.Pi{i})));
    chainElement.appendChild(thisElement);
    
    thisElement = docNode.createElement('ObservationUNL');
    thisElement.appendChild(docNode.createTextNode(sprintf('%s',observationUNLs{i})));
    chainElement.appendChild(thisElement);
    
    thisElement = docNode.createElement('StateUNL');
    thisElement.appendChild(docNode.createTextNode(sprintf('%s',outputUNLs{i})));
    chainElement.appendChild(thisElement);
    
    if(bnet.o_type(i) == 'c')
        
        continuousElement = docNode.createElement('B_Continuous');
        chainElement.appendChild(continuousElement);
        
        for n=1:bnet.n_states(i)

            classElement = docNode.createElement('Gaussian');
            continuousElement.appendChild(classElement);

            thisElement = docNode.createElement('Label');
            thisElement.appendChild(docNode.createTextNode(sprintf('%s',stateStrings{i}{n})));
            classElement.appendChild(thisElement);

            thisElement = docNode.createElement('Det');
            thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g',det(bnet.cov{i}(:,:,n)))));
            classElement.appendChild(thisElement);

            thisElement = docNode.createElement('Mean');
            thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',bnet.B{i}(n,:))));
            classElement.appendChild(thisElement);

            matrixElement = docNode.createElement('Covariance');
            classElement.appendChild(matrixElement);

            thisElement = docNode.createElement('NumRows');
            thisElement.appendChild(docNode.createTextNode(sprintf('%d',bnet.m_outputs(i))));
            matrixElement.appendChild(thisElement);
            thisElement = docNode.createElement('NumCols');
            thisElement.appendChild(docNode.createTextNode(sprintf('%d',bnet.m_outputs(i))));
            matrixElement.appendChild(thisElement);
            %inv_cov = bnet.cov{i}(:,:,n)^(-1);  %for the IM, we want cov; for GMM, inv_cov
            cov = bnet.cov{i}(:,:,n);
            for m=1:bnet.m_outputs(i)
                thisElement = docNode.createElement('Row');
                thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',cov(m,:))));
                matrixElement.appendChild(thisElement);
            end
        end
    elseif (bnet.o_type(i) == 'd')
        matrixElement = docNode.createElement('B_Discrete');
        chainElement.appendChild(matrixElement);

        thisElement = docNode.createElement('NumRows');
        thisElement.appendChild(docNode.createTextNode(sprintf('%d',bnet.n_states(i))));
        matrixElement.appendChild(thisElement);
        thisElement = docNode.createElement('NumCols');
        thisElement.appendChild(docNode.createTextNode(sprintf('%d',bnet.m_outputs(i))));
        matrixElement.appendChild(thisElement);
        for m=1:bnet.n_states(i)
            thisElement = docNode.createElement('Row');
            thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',bnet.B{i}(m,:))));
            matrixElement.appendChild(thisElement);

        end

    end
end

matrixElement = docNode.createElement('D');
docRootNode.appendChild(matrixElement);

thisElement = docNode.createElement('NumRows');
thisElement.appendChild(docNode.createTextNode(sprintf('%d',sum(bnet.nchains))));
matrixElement.appendChild(thisElement);
thisElement = docNode.createElement('NumCols');
thisElement.appendChild(docNode.createTextNode(sprintf('%d',sum(bnet.nchains))));
matrixElement.appendChild(thisElement);
for k=1:sum(bnet.nchains)
    thisElement = docNode.createElement('Row');
    thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',bnet.T(k,:))));
    matrixElement.appendChild(thisElement);
end

A_Element = docNode.createElement('A');
docRootNode.appendChild(A_Element);

for i=1:bnet.nchains
    for j=1:bnet.nchains
        A_subElement = docNode.createElement('A_sub');
        A_Element.appendChild(A_subElement);
        thisElement = docNode.createElement('c1');
        thisElement.appendChild(docNode.createTextNode(sprintf('%d',i)));
        A_subElement.appendChild(thisElement);
        thisElement = docNode.createElement('c2');
        thisElement.appendChild(docNode.createTextNode(sprintf('%d',j)));
        A_subElement.appendChild(thisElement);
        
        matrixElement = docNode.createElement('TransitionMatrix');
        A_subElement.appendChild(matrixElement);
        thisElement = docNode.createElement('NumRows');
        thisElement.appendChild(docNode.createTextNode(sprintf('%d',bnet.n_states(i))));
        matrixElement.appendChild(thisElement);
        thisElement = docNode.createElement('NumCols');
        thisElement.appendChild(docNode.createTextNode(sprintf('%d',bnet.n_states(j))));
        matrixElement.appendChild(thisElement);
        for k=1:sum(bnet.n_states(i))
            thisElement = docNode.createElement('Row');
            %actually correct:
            thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',bnet.A{i,j}(k,:))));
            %bugfix: enchant::Matrix uses column-order representation in Memory, so we transpose here
            %thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',bnet.A{i,j}(:,k))));
            matrixElement.appendChild(thisElement);
        end
    end
end

% This is not required. H is fully specified by D and A and is internally
% computed in the c-code for the influence model. It is however nice to visually include H 
H = influence_matrix(bnet);
matrixElement = docNode.createElement('H');
docRootNode.appendChild(matrixElement);
matrixElement.appendChild(docNode.createComment('InfluenceMatrix H, computable from D and A, thus only for eye-candy'));
thisElement = docNode.createElement('NumRows');
thisElement.appendChild(docNode.createTextNode(sprintf('%d',size(H,1))));
matrixElement.appendChild(thisElement);
thisElement = docNode.createElement('NumCols');
thisElement.appendChild(docNode.createTextNode(sprintf('%d',size(H,2))));
matrixElement.appendChild(thisElement);
for k=1:size(H,1)
    thisElement = docNode.createElement('Row');
    thisElement.appendChild(docNode.createTextNode(sprintf('%-1.30g ',H(k,:))));
    matrixElement.appendChild(thisElement);
end

%%%%% Save the XML document.
xmlwrite(fileName,docNode);
edit(fileName);

fprintf('done.\n');
