function [accuracy confusion t_classify] = classifyBNET(features, labels, C)
%
% [accuracy confusion t_classify] = classifyBNET(features, labels, C)
%
% Note: this code was used for trying out. 
%       The final version is called GClassifier
%


%%% This was the code for classification with the Bnet toolbox.
%%% It was slow, so this simple gaussian classifier is coded below
%
% evidence=cell(2,1);   %% Start out with nothing observed
% engine = jtree_inf_engine(C.bnet);
% fprintf('Classifying test data using find_mpe()');
% numSlices = size(features,1);
% tic;
% for i=1:numSlices
%     if mod(i,numSlices/10)==0, fprintf(' %2.0f%%',100/numSlices*i); end
%     evidence{2}=features(i,:)';
%     [engine2, ll] = enter_evidence(engine,evidence);
%     marg = marginal_nodes(engine2,1);
%     p(i,:)=marg.T';
%     %mpe = find_mpe(engine, evidence);
%     %decision(i,:) = mpe{1}(1);
%     [foo decision(i,:)] = max(p(i,:));
% end

tic
priors = get_field(C.bnet.CPD{1},'cpt');
cv = get_field(C.bnet.CPD{2},'cov');
u = get_field(C.bnet.CPD{2},'mean');
numClasses = size(u,2);
numFeatures = size(features,2);
numSlices = size(features,1);
fprintf('Classifying test data');
% pre-computation of expensive calculations that are the same for all cases
for j=1:numClasses
    sigma_det(j) = det(cv(:,:,j));
    inv_cov(:,:,j) = cv(:,:,j)^(-1);
end
% for j=1:numClasses       %not done for E-step
%    cv(:,:,j) = diag(ones(1,numFeatures).*0.01);
% end
for i=1:numSlices
    if mod(i,floor(numSlices/10))==0, fprintf(' %2.0f%%',100/numSlices*i); end
    summ = 0;
    for j=1:numClasses,
         p(i,j) = priors(j) / sqrt( (2*pi)^numFeatures * sigma_det(j))*exp(-0.5*(features(i,:)-u(:,j)')*inv_cov(:,:,j)*(features(i,:)-u(:,j)')');
         summ = summ + p(i,j);
    end
    for j=1:numClasses,
         p(i,j) = p(i,j) / summ;
    end
    [foo decision(i,:)] = max(p(i,:));
end

t_classify = toc;
fprintf('\nClassifying time with feature type ''%s'' was %3.1f seconds.\n',C.featureType,t_classify);
drawnow;
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
subplot(2,1,1);
%plot(features);
hold on;
plot(decision(:,1)*10,'b--');
plot(labels*10,'k--');
title(sprintf('Test data with esimates (blue dashed) and ground truth (black dashed) for ''%s''',C.featureType));
subplot(2,1,2);
hold on;
plot(p); drawnow;
correctClassifications = find(decision(:,1)==labels);
wrongClassifications = find(decision(:,1)~=labels);
plot(correctClassifications,ones(1,length(correctClassifications))*0.5,'g.');
plot(wrongClassifications,ones(1,length(wrongClassifications))*0.4,'r.');
title('Probability of the different classes. Green and red are correct and wrong classifications.');
drawnow;

%% Now calculate the accuracy: correct/total * 100%
accuracy=100*sum(decision(:,1)==labels)/length(labels);
fprintf('Accuracy with feature type ''%s'' was %3.1f%%.\n',C.featureType,accuracy);
drawnow;

% compute confusion matrix
confusion = zeros(C.NUM_CLASSES, C.NUM_CLASSES);
for k=1: C.NUM_CLASSES
    for j=1: C.NUM_CLASSES
     index = find(labels == k);
      confusion(k,j) = length(find(decision(index,:) == j));
    end
end
%confusion = confusion ./ length(test);
confusion

