function [TrafoMatrix, features_afterLDA] = computeLDA(features, labels, varargin)
%
% [features_afterLDA, TrafoMatrix] = computeLDA(features, labels)
% 
% Computes a Linear Discriminent Analysis on a set of labelled features.
% The transformed data is plot into a 3-D feature-space.
%
% labels are assumed as consecutive integeres starting from 1,2,...

PLOTSPACE = 3;
if (nargin >= 1) PLOTSPACE = varargin{1}; end

NUM_CLASSES = max(labels);
[nCases nDim] = size(features);

Means = zeros(NUM_CLASSES, nDim);

for i=1:NUM_CLASSES
	Means(i,:) = mean(features(find(labels==i),:),1);
end

Between = cov(Means);
Within  = cov(features - Means(labels,:));

temp = Within \ Between;
if (find(isnan(temp)==1))
    TrafoMatrix = -1;
    features_afterLDA = -1;
    fprintf('ComputeLDA: Error, NaN prevents convergence, LDA exits!\n');
    return;
end

[v d] = eig(temp);   % eig( Sw^(-1) * Sb )

%disp(diag(d))

[Sorted Order] = sort(diag(d));

Sorted2 = flipud(Sorted);
Order2  = flipud(Order);

fprintf('Sorted EigenValues:\n');
disp(Sorted2')

%Order2NonZero = Order2(find(abs(Sorted2)>0.00001));
Order2NonZero = Order2(1:PLOTSPACE);

%LDA_Eig_Values = Sorted2(1:5)
OrderedEigs = v(:,Order2NonZero);

features_afterLDA = features * OrderedEigs;
TrafoMatrix = OrderedEigs;

%test_features_afterLDA = test_features * TrafoMatrix;
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
color = {'b.','go','rx','c+','m*','ks','yd','gp','rh','r','c','m','y','k'};
if (PLOTSPACE == 3)
for i=1:NUM_CLASSES
    X=features_afterLDA(find(labels==i),1);
    Y=features_afterLDA(find(labels==i),2);
    Z=features_afterLDA(find(labels==i),3);
    plot3(X,Y,Z,color{i}); hold on; grid on;
end
title('3D-parameter space of features');
end
if (PLOTSPACE == 2)
for i=1:NUM_CLASSES
    X=features_afterLDA(find(labels==i),1);
    Y=features_afterLDA(find(labels==i),2);
    plot(X,Y,color{i}); hold on; grid on;
end
title('2D-parameter space of features');
end
hold off;
return