function [labelCount clipIndex notClipIndex] = findLabels(clips,contextDimension,contextClasses,verbose)
%
% [labelCount clipIndex notClipIndex] = findLabels(clips,contextDimension,contextClasses,verbose)
%
% clips : cell array of clips
% contextDimension : [1..4] integer corresponding to the index of the dimension of interest in clips.label
% contextClasses : array of integers with the class labels that shall be selected [0..n], [1..n] for speech
%
% labelcount : array showing how many labels of each class were found in the dimension given
% clipIndex : the indices of the clips that are labeled as one of the context classes
% notClipIndex : indices of clips that are NOT in those classes (the others)
%

if not(exist('verbose'))
    verbose = 1;
end

labelCount = zeros(1,10);
clipIndex = []; notClipIndex = [];
if not(exist('contextClasses'))
    contextClasses = -2;
end
contextClasses = contextClasses + 1;
count = 1; notCount = 1;
for i=1:length(clips)
    currentLabel = clips{i}.label(contextDimension+1) + 1;  % '+1' accounts for unknown, which is label 0
    if (currentLabel > 10)
        printf('Error, currentLabel > 10, edit findLabels.m\n');
        return;
    end
    if find(currentLabel == contextClasses)
        clipIndex(count) = i;
        count = count + 1;
    else
        notClipIndex(notCount) = i;
        notCount = notCount + 1;
    end
    labelCount(currentLabel) = labelCount(currentLabel) + 1;
end

% LABELS{1} = {'Dont know','Office','Home','Outdoors','Indoors','Subway','Car','Street','Shop','Restautant'};
% LABELS{2} = {'Dont know','No speech','Me Speaking','Other speaker','Distant voices','Loud crowd','Speech, but none of the above'};
% LABELS{3} = {'Dont know','Lying  ','Sitting','Standing','Walking','Running','Biking '};
% LABELS{4} = {'Dont know','Eating','Typing','Shaking hands','Clapping hands','Driving','Brushing teeth','Doing the dishes','Manipulation'};

% reduced, for Mark's thesis
LABELS{1} = {'Dont know','Office','Home','Outdoors','Indoors','Restautant','Car','Street','Shop','former restaurant'};
LABELS{2} = {'Dont know','No speech','Me Speaking','Other speaker','Distant voices','Loud crowd','Speech, but none of the above'};
LABELS{3} = {'Dont know','Lying  ','Sitting','Standing','Walking','Running','Biking '};
LABELS{4} = {'Dont know','Eating','Typing','Shaking hands','Clapping hands','Driving','Brushing teeth','Doing the dishes','Manipulation'};


if (verbose)
    set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
    bar(labelCount);
    title('Distribution of labels over all clips');
    a = axis(); axis([a(1) a(2) a(3) a(4)*1.1]);
    for c=1:length(LABELS{contextDimension})
        H = text(c-0.5,a(4)*1.05,LABELS{contextDimension}{c});
        set(H,'FontSize',8);
    end

    %%%% count time for each class
    timeCount = zeros(1,length(LABELS{contextDimension}));
    for i=1:length(clips)
        currentLabel = clips{i}.label(contextDimension+1) + 1;
        if (contextDimension < 3) %audio only
            timeCount(currentLabel) = timeCount(currentLabel) + (clips{i}.audioStop - clips{i}.audioStart);
        else
            timeCount(currentLabel) = timeCount(currentLabel) + (clips{i}.acc1(end,1) - clips{i}.acc1(1,1));
        end
    end

    for c=1:length(LABELS{contextDimension})
        fprintf('%s:\t%3.0f seconds =\t%3.1f minutes =\t%2.2f hours\n',LABELS{contextDimension}{c}, timeCount(c), timeCount(c)/60, timeCount(c)/3600);
    end

end



