
function [engine, ll] = enter_evidence(engine, evidence, states)

% ENTER_EVIDENCE --- calculate sufficient statistics from ev
% since two_slides_evidence might be very large,
% instead of choosing one_slide_evidence and two_slides_evidence
% as sufficient statistics, we use alpha, beta, and obslik .

clamp_chains = 1;

bnet = engine.bnet;
ns = bnet.n_states;
m = bnet.m_outputs;
T = size(evidence,2);
prior = cat(2,bnet.Pi{:});
influence = influence_matrix(bnet);

%
s_ns = sum(ns);
l_ns = length(ns);
ns_1 = ones(s_ns,1);

ndx = [0 cumsum(ns)];
if nargin<3, states = nan(l_ns,T); end
latent = nan(s_ns,T);
t0 = 1:T;
for i=1:l_ns
  latent(ndx(i)+1:ndx(i+1),~isnan(states(i,:))) = 0;
  ndx0 = ndx(i)+states(i,:);
  ndx1 = sub2ind( [s_ns,T], ndx0(~isnan(ndx0)), t0(~isnan(ndx0)) );
  latent( ndx1 ) = 1;
end

obslik = zeros(s_ns,T);
for i=1:l_ns
  ev = cell2num(evidence(i,:));
  ndx0 = ~isnan(sum(ev,1));
  if bnet.o_type(i) == 'd'
    obslik(ndx(i)+1:ndx(i+1),ndx0)=multinomial_prob(ev(:,ndx0),bnet.B{i});
  else
    for j=1:ns(i)
      M = bnet.B{i}(j,:)';
      C = bnet.cov{i}(:,:,j);
      obslik(ndx(i)+j,ndx0)=gaussian_prob(ev(:,ndx0),M,C)';
    end
  end
end
obslik(~isnan(latent)) = latent(~isnan(latent));
obslik = obslik';
obslik = obslik + 1e-30; % hack ! trying to avoid divide by 0

ndx9 = zeros(s_ns);
for i=1:l_ns
  ndx9(ndx(i)+1:ndx(i+1),ndx(i)+1:ndx(i+1))=1;
end
ndx9 = sparse(ndx9);
if ~clamp_chains, scale = ones(1, T);
else
  scale = ones(T,s_ns);
end

alpha = zeros(T,s_ns);
alpha(1,:) = prior.*obslik(1,:);

if ~clamp_chains
  scale(1) = sum(alpha(1,:));
  alpha(1,:) = alpha(1,:)/scale(1);
else
  scale(1,:) = alpha(1,:)*ndx9;
  alpha(1,:) = alpha(1,:)./scale(1,:);
end

for t=2:T
  alpha(t,:) = alpha(t-1,:)*influence.*obslik(t,:);
  if ~clamp_chains
    scale(t) = sum(alpha(t,:));
    alpha(t,:) = alpha(t,:)/scale(t);
  else
    scale(t,:) = alpha(t,:)*ndx9;
    alpha(t,:) = alpha(t,:)./scale(t,:);
  end
end

if ~clamp_chains
  Pc0 = zeros(1,l_ns);
  for i=1:l_ns
    Pc0(i) = sum(alpha(T,ndx(i)+1:ndx(i+1)));
  end
  ll=sum(log(scale))*l_ns+sum(log(Pc0));
else, ll = sum(sum(log(scale(:,ndx(1:end-1)+1))));
end

beta = zeros(s_ns,T);
for i=1:l_ns
  beta(ndx(i)+1:ndx(i+1),T) = 1/sum(alpha(T,ndx(i)+1:ndx(i+1)));
  beta(alpha(T,:)==0,T) = 0;
end
for t=T-1:-1:1
  if ~clamp_chains
    beta(:,t) = influence*(obslik(t+1,:)'.*beta(:,t+1))/scale(t+1);
  else
    beta(:,t) = influence*(obslik(t+1,:)'.*beta(:,t+1)./scale(t+1,:)');
  end
end
beta(:) = 1;  % temporarily ONLY use forward algorithm

engine.alpha = alpha;
engine.beta = beta;
engine.obslik = obslik;
engine.scale = scale;
