
function [B,D1,D2,pinvB] = event_matrix(bnet)

% EVENT_MATRIX generates the event matrix B
% from the influence model representation
% refer Chalee's thesis

nc = bnet.nchains;
ns = bnet.n_states;

ndx = [1 cumsum(ns) + 1];
st3p = cumprod(ns(nc:-1:1));
st3p = [st3p(nc-1:-1:1) 1];
sz1 = prod(ns);
sz = sz1 ./ ns;
sz2 = sum(ns);
B = zeros(sz1, sz2);

for c = 1:nc
  fil1 = eye(ns(c));
  fil1 = fil1( :,:,ones(1,sz(c)) );
  fil1 = reshape(fil1, [ns(c) ns(c) st3p(c) sz(c)/st3p(c)]); 
  fil1 = permute(fil1, [3 1 4 2]);
  fil1 = reshape(fil1, [sz1 ns(c)]);
  B(:, ndx(c):(ndx(c)+ns(c)-1)) = fil1;
end

if nargout~=4, return, end

D1 = ones(1,sz2);
D2 = zeros(1,sz2);
denon = 0;
for c=1:nc
  D1(ndx(c):ndx(c)+ns(c)-1) = 1+ns(c)*sum(1./ns([1:c-1 c+1:nc]));
  D2(ndx(c):ndx(c)+ns(c)-1) = -sum(1./ns([1:c-1 c+1:nc]));
  denon = denon+prod(ns([1:c-1 c+1:nc]));
end
D1 = D1./denon;
D2 = D2./denon;
pinvB = (diag(D1)*B'+diag(D2)*ones(sz2,sz1));