function bnet = maximize_params(engine, evidence)

% MAXIMIZE_PARAMS maximize dbn parameters according to 
% sufficient statistics we collected

bnet = engine.bnet;
ns = bnet.n_states;
m = bnet.m_outputs;
T = size(evidence,2);
influence = influence_matrix(bnet);

%
s_ns = sum(ns);
l_ns = length(ns);
ns_1 = ones(s_ns,1);

alpha = engine.alpha;
beta = engine.beta;
obslik = engine.obslik;
scale = engine.scale;

ndx = [0 cumsum(ns)];
ndx9 = zeros(s_ns);
for i=1:l_ns
  ndx9(ndx(i)+1:ndx(i+1),ndx(i)+1:ndx(i+1))=1;
end
ndx9 = sparse(ndx9);

gamma = alpha.*beta'; % one slice marginal
if size(scale,1)==1
  coeff = obslik./scale(ns_1,:)';  
else   
  coeff = obslik./scale;
end

pi = gamma(1,:)./(gamma(1,:)*ndx9);
for c1=1:l_ns
  engine.bnet.Pi{c1} = pi(ndx(c1)+1:ndx(c1+1));
end

mem_tol = 1024*1024; 
A = zeros(s_ns);
chunk_sz = s_ns^2;
step_sz = floor(mem_tol/chunk_sz);

for t0 = 1:step_sz:T-1
  t1 = min(t0+step_sz-1,T-1); t=t0:t1;
  xi = permute(alpha(t,:,ns_1),[2 3 1]).*...
       influence(:,:,ones(t1-t0+1,1)).*...
       permute(coeff(t+1,:,ns_1),[3 2 1]).*...
       permute(beta(:,t+1,ns_1),[3 1 2]); % two slice marginal
  A = A+sum(xi,3);
end

for c1=1:l_ns
  for c2=1:l_ns
     engine.bnet.A{c1,c2} = ...
         normalize(A(ndx(c1)+1:ndx(c1+1),ndx(c2)+1:ndx(c2+1)),2);
  end
end

coeff2 = zeros(l_ns, s_ns);
for i=1:l_ns,coeff2(i,ndx(i)+1:ndx(i+1))=1;end
engine.bnet.T = normalize(log(1+(2*exp(1)-1)*coeff2*A*coeff2'),1);

cov_prior = .01;
coeff2 = 1:max(ns); coeff2 = coeff2(ones(T,1),:);
for i=1:l_ns
  ev = cell2num(evidence(i,:));
  ndx0 = ~isnan(sum(ev,1));
  if bnet.o_type == 'd'
    engine.bnet.B{i} = full(sparse(...
        coeff2(ndx0,1:ns(i)),ev(ones(ns(i),1),ndx0)',...
        gamma(ndx0,ndx(i)+1:ndx(i+1)),ns(i),m(i)));
    engine.bnet.B{i} = normalize(engine.bnet.B{i},2);
  else
    for j=1:ns(i)
      coeff=gamma(:,ndx(i)+j)'; coeff=coeff/sum(coeff(ndx0));
      m_1 = ones(m(i),1);
      engine.bnet.B{i}(j,:) = sum(coeff(ones(m(i),1),ndx0).*ev(:,ndx0),2)';
      
      engine.bnet.cov{i}(:,:,j) = zeros(m(i));
      chunk_sz = m(i)^2;
      step_sz = floor(mem_tol/chunk_sz);
      for t0 = 1:step_sz:T
        t1 = min(t0+step_sz-1,T); 
        t=t0:t1; t=t(ndx0(t));
        engine.bnet.cov{i}(:,:,j) = engine.bnet.cov{i}(:,:,j)+sum(...
            permute(ev(:,t,m_1),[1 3 2]).*permute(ev(:,t,m_1),[3 1 2]).*...
            reshape(coeff(ones(m(i)),t),[m(i) m(i) size(t,2)]),3); 
      end
      engine.bnet.cov{i}(:,:,j) = engine.bnet.cov{i}(:,:,j) - ...
          engine.bnet.B{i}(j,:)'*engine.bnet.B{i}(j,:);
      % hack, add some prior
      engine.bnet.cov{i}(:,:,j)=engine.bnet.cov{i}(:,:,j)+cov_prior*eye(m(i));
    end
  end
end

bnet = engine.bnet;
