function bnet = mk_influence(n_states, m_outputs, o_type)

% MK_INFLUENCE --- construct an influence model dbn
%
% input:
% n_states --- number of latent states for each chain (1 x n_chains) 
% m_outputs --- for discrete output,  number of output symbols
%               for continous output, dimensionality  (1 x n_chains)
% o_type --- output type, 'c' continuous, 'd' discrete (1 x n_chains)
% the chains can have different # of latent states and output
%
% n_states contains # of internal states for each chain

if nargin<3, o_type = repmat('d',1,length(n_states));end

bnet.nchains = length(n_states);
bnet.n_states = n_states;
bnet.m_outputs = m_outputs;
bnet.o_type = o_type;

% initial distribution for each hidden node in each. C sites
bnet.Pi = cell(1, bnet.nchains);
for i=1:bnet.nchains
  bnet.Pi{i} = mk_stochastic(rand(1, n_states(i)));
end

% output matrix for each output node in each. C sites
bnet.B = cell(1, bnet.nchains);
bnet.cov = cell(1, bnet.nchains);
for i=1:bnet.nchains
  if o_type(i) == 'd'
    bnet.B{i} = mk_stochastic(rand(n_states(i), m_outputs(i)));
  elseif o_type(i) == 'c'
    bnet.B{i} = randn(bnet.n_states(i),bnet.m_outputs(i));
    bnet.cov{i} = 100*repmat(eye(bnet.m_outputs(i)),[1 1 bnet.n_states(i)]);
  else
    error 'invalid output';
  end
end

% C^2 transition matrix for every 2 hidden nodes in different sites
% A is 4-dimensional: (site m node i, site n node j)
bnet.A = cell(bnet.nchains, bnet.nchains);
for i=1:bnet.nchains
  for j=1:bnet.nchains
    bnet.A{i,j} = mk_stochastic(rand(n_states(i),n_states(j)));
  end
end

% C coupling parameters for every 2 sites
bnet.T = mk_stochastic(rand(bnet.nchains))';

