function seq = sample_influence(bnet, T)

% SAMPLE_INFLUENCE generates a random sequence from bnet
% seq{1,x,y} are states of hidden nodes at site x and slice y
% seq{2,x,y} are symbols observable nodes at site x and slice y

seq = cell(2, bnet.nchains, T);

% sample the first slice
t = 1;
for i=1:bnet.nchains
  seq{1,i,t} = sample_discrete(bnet.Pi{i});
  if bnet.o_type == 'd'
    B = bnet.B{i}(seq{1,i,t},:);
    seq{2,i,t} = sample_discrete(B);
  else
    B = bnet.B{i}(seq{1,i,t},:); % mean
    c = bnet.cov{i}(:,:,seq{1,i,t}); % cov
    seq{2,i,t} = sample_gaussian(B,c)';
  end
end

% sample the rest slices
for t=2:T
  for i=1:bnet.nchains
    % latent state
    T0 = zeros(1, bnet.n_states(i));
    for k=1:bnet.nchains
      A = bnet.A{k,i};
      T0=T0+bnet.T(k,i)*A(seq{1,k,t-1},:);
    end
    seq{1,i,t} = sample_discrete(T0);
    % output
    if bnet.o_type == 'd'
      B = bnet.B{i}(seq{1,i,t},:);
      seq{2,i,t} = sample_discrete(B);
    else
      B = bnet.B{i}(seq{1,i,t},:); % mean
      c = bnet.cov{i}(:,:,seq{1,i,t}); % cov
      seq{2,i,t} = sample_gaussian(B,c)';
    end
  end
end
