function plotClips(clips,sensor)
% Version 1.1
% plotClips(clips,sensor)
%
% Creates a new full size figure containing a plot of all the data
% specified in sensor, with some of the labels
% 'clips' is a cell array of clips, generated with SignalLoader()
% 'sensor' can have the values 'acc1' 'acc2' 'acc1_acc2' 'audio'
% It is assumed, that the order of the clips within a dataset are
% in chronological order

if not(exist('sensor'))
    fprintf('plotClips: Warning, no sensor argument, assuming acc1\n')
    %error(['argument ''sensor'' is not defined!']); return;
    sensor = 'acc1';
end

chainStrings = {'location','speech ','posture','events '};
stateStrings = {{'unknown','office','home','outdoors','indoors','subway','car','street','shop','restautant'},...
                {'unknown','no speech','me speaking','other speaker','distant voices','loud crowd','other speech'},...
                {'unknown','lie','sit','stand','walk','run','bike'},...
                {'no event','eating','typing','shaking hands','clapping hands','driving','brushing teeth','doing the dishes','manipulation'}};
colors = {'b', 'g', 'r', 'c', 'm', 'y', 'k','b', 'g', 'r', 'c', 'm', 'y', 'k','b', 'g', 'r', 'c', 'm', 'y', 'k',};
    
set(figure,'Color','white','Position',[5 29 1270 920]);
hold on;
previousDataset = '';
timeAcc1 = [0];  timeAcc2 = [0]; 
previousAudioStop = [0];
k = 0;
legendStrings = {};
for i=1:length(clips)
    switch sensor,
        case 'acc1', 
            if (not(strcmp(clips{i}.dataset,previousDataset)))
                previousDatasetEndTime = timeAcc1(end);
                k = k+1; legendStrings{k} = clips{i}.dataset;
            end
            timeAcc1 = clips{i}.acc1(:,1) + previousDatasetEndTime;
            plot(timeAcc1,clips{i}.acc1(:,2:4));
            clipStartTime = timeAcc1(1);

        case 'acc2', 
            if (not(strcmp(clips{i}.dataset,previousDataset)))
                previousDatasetEndTime = timeAcc2(end);
                k = k+1; legendStrings{k} = clips{i}.dataset;
            end
            timeAcc2 = clips{i}.acc2(:,1) + previousDatasetEndTime;
            plot(timeAcc2,clips{i}.acc2(:,2:4));
            clipStartTime = timeAcc2(1);
            
        case 'acc1_acc2',
            if (not(strcmp(clips{i}.dataset,previousDataset)))
                previousDatasetEndTime = max(timeAcc1(end),timeAcc2(end));
                k = k+1; legendStrings{k} = clips{i}.dataset;
            end
            timeAcc1 = clips{i}.acc1(:,1) + previousDatasetEndTime;
            timeAcc2 = clips{i}.acc2(:,1) + previousDatasetEndTime;
            plot(timeAcc1,clips{i}.acc1(:,2:4), timeAcc2,clips{i}.acc2(:,2:4));
            clipStartTime = min(timeAcc1(1),timeAcc2(1));
            
        case 'audio', 
            if (not(strcmp(clips{i}.dataset,previousDataset)))
                previousDatasetEndTime = previousAudioStop;
                k = k+1; legendStrings{k} = clips{i}.dataset;
            end
            plot(linspace(previousDatasetEndTime+clips{i}.audioStart,previousDatasetEndTime+clips{i}.audioStop,...
                length(clips{i}.audio)),double(clips{i}.audio)/2^15);
            clipStartTime = previousDatasetEndTime + clips{i}.audioStart;
            previousAudioStop = clips{i}.audioStop;
            
        otherwise,
            error(['invalid sensor option ' sensor]);
    end
    
    a = axis;
    H = text(clipStartTime, a(4)-(a(4)-a(3))/9, sprintf('%d\n%s\n%s\n%s\n%s',i,...
        stateStrings{1}{clips{i}.label(2)+1},stateStrings{2}{clips{i}.label(3)+1},...
        stateStrings{3}{clips{i}.label(4)+1},stateStrings{4}{clips{i}.label(5)+1}));
    set(H,'FontSize',8,'Color',colors{k});
    previousDataset = clips{i}.dataset;
end
title('plotClips()');
[LEGH,OBJH,OUTH,OUTM] = legend(legendStrings,'Location','SouthEast'); %legend boxoff;
set(OBJH(1:k),'FontSize',8);   for i=1:k, set(OBJH(k+(i-1)*2+1),'Color',colors{i}); end
    
hold off;
drawnow;