%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Written by Mark Blum (mark@media.mit.edu) in September 2005
%%%
%%% This script loads data recording by SignalRecorder in the ASSIST
%%% configuration, with the AWare sensor box. All recordings start 
%%% and end with clapping of hands. This script calculates the time-
%%% stamp difference using cross-correlation of the audio signals.

close all; drawnow;

DRIVE_LETTER = 'E';
% loadNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\assist\mark1\Mark1_z6000_12_09_30');
% loadNamePrefixAware = strcat(DRIVE_LETTER,':\DiaryAppData\assist\mark1\mark1');
loadNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\assist\mark2\mark2_z6000_12_37_41');
loadNamePrefixAware = strcat(DRIVE_LETTER,':\DiaryAppData\assist\mark2\mark2_z6000_12_37_41');

fprintf('loading acceleration data...');
acc1 = load([loadNamePrefix '.accel1']);
acc2 = load([loadNamePrefix '.accel2']);
awareAcc = load([loadNamePrefixAware '.awareAccel']);
fprintf('done\n');


%%%%% Load Zaurus audio %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
audioSamplesPerLine = 1024;
fileName = [loadNamePrefix '.audio'];
fprintf('loading %s\n',fileName);
%%% we load the file by splitting off the time-stamp into a double
%%% float, and the samples into d16 (signed short) in order to save
%%% memory. textscan is said to be fast
fprintf('    Reading TimeStamps and determining number of lines in file: ');
fid = fopen(fileName,'r');
audio.timeStamps = cell2num(textscan(fid,'%f64%*[^\n]','delimiter','EOL','bufSize',audioSamplesPerLine*8));
fclose(fid);
numRows = size(audio.timeStamps,1);
fprintf('%d\n',numRows);
fprintf('    Allocating memory (%4.0f MB)\n',numRows*audioSamplesPerLine*2/1000000);
audio.values = zeros(numRows,audioSamplesPerLine,'int16');
format = '%*s'; %skip timestamp
for i=1:audioSamplesPerLine,
    format = sprintf('%s %%d16',format);
end
fid = fopen(fileName,'r');
ROWS_PER_CHUNK = 10000;
fprintf('    Reading values: ');
for i=1:ceil(numRows/ROWS_PER_CHUNK);
    fprintf('%2.0f%% ',((i-1)/ceil(numRows/ROWS_PER_CHUNK)*100));
    audio.values((i-1)*ROWS_PER_CHUNK+1:min(i*ROWS_PER_CHUNK,numRows),:) = cell2mat(textscan(fid,format,ROWS_PER_CHUNK,'bufSize',audioSamplesPerLine*8));
end
fprintf('100%% \n');
fclose(fid);

%%%%% Load Aware audio %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
audioSamplesPerLine = 16;
fileName = [loadNamePrefixAware '.awareAudio'];
fprintf('loading %s\n',fileName);
fprintf('    Reading TimeStamps and determining number of lines in file: ');
fid = fopen(fileName,'r');
awareAudio.timeStamps = cell2num(textscan(fid,'%f64%*[^\n]','delimiter','EOL','bufSize',audioSamplesPerLine*8));
fclose(fid);
numRows = size(awareAudio.timeStamps,1);
fprintf('%d\n',numRows);
fprintf('    Allocating memory (%4.0f MB)\n',numRows*audioSamplesPerLine*2/1000000);
awareAudio.values = zeros(numRows,audioSamplesPerLine,'int16');
format = '%*s'; %skip timestamp
for i=1:audioSamplesPerLine,
    format = sprintf('%s %%d16',format);
end
fid = fopen(fileName,'r');
ROWS_PER_CHUNK = 200000;
fprintf('    Reading values: ');
for i=1:ceil(numRows/ROWS_PER_CHUNK);
    fprintf('%2.0f%% ',((i-1)/ceil(numRows/ROWS_PER_CHUNK)*100));
    awareAudio.values((i-1)*ROWS_PER_CHUNK+1:min(i*ROWS_PER_CHUNK,numRows),:) = cell2mat(textscan(fid,format,ROWS_PER_CHUNK,'bufSize',audioSamplesPerLine*8));
end
fprintf('100%% \n');
fclose(fid);

%create nicer timestamp values
t0 = min([audio.timeStamps(1),awareAudio.timeStamps(1),acc1(1,1),acc2(1,1),awareAcc(1,1)]);
audio.timeStamps = audio.timeStamps - t0;
awareAudio.timeStamps = awareAudio.timeStamps - t0;
acc1(:,1) = acc1(:,1) - t0;
acc2(:,1) = acc2(:,1) - t0;
awareAcc(:,1) = awareAcc(:,1) - t0;

%%%%% reshape audio data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
audioSamplesPerLine = 1024;
numSamples = size(audio.timeStamps,1)*audioSamplesPerLine;
audio.start = audio.timeStamps(1)-1024/8000;
audio.stop = audio.timeStamps(end);
audio.rate = numSamples/(audio.stop - audio.start);
fprintf('Zaurus audio: numSamples = %d, sampleRate = %g\n',numSamples,audio.rate);
audio.values = reshape(audio.values',numSamples,1);

audioSamplesPerLine = 16;
numSamples = size(awareAudio.timeStamps,1)*audioSamplesPerLine;
awareAudio.start = awareAudio.timeStamps(1)-16/8000;
awareAudio.stop = awareAudio.timeStamps(end);
awareAudio.rate = numSamples/(awareAudio.stop - awareAudio.start);
fprintf('AWare audio: numSamples = %d, sampleRate = %g\n',numSamples,awareAudio.rate);
awareAudio.values = reshape(awareAudio.values',numSamples,1);



% set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
% subplot(2,1,1); 
% plot(awareAcc(:,1),awareAcc(:,2:4)); title('Aware accelerometer');
% subplot(2,1,2); 
% plot(awareAudio); title('Aware Audio');

% set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
% subplot(3,1,1); 
% plot(awareAcc(:,1),awareAcc(:,2:4)); title('Aware accelerometer');
% subplot(3,1,2);
% plot(acc1(:,1),acc1(:,2:4));  title('MITe1');
% subplot(3,1,3);
% plot(acc2(:,1),acc2(:,2:4));  title('MITe2');

set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
subplot(3,1,1); 
plot(awareAcc(:,2:4)); title('Aware accelerometer');
subplot(3,1,2);
plot(acc1(:,2:4));  title('MITe1');
subplot(3,1,3);
plot(acc2(:,2:4));  title('MITe2');

% set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
% subplot(2,1,1); 
% plot(awareAudio.values(1:16:end)); title('Aware audio');
% subplot(2,1,2); 
% plot(audio.values(1:16:end)); title('Zaurus audio');

% audioNorm = audio/max(audio);
% awareAudioNorm = awareAudio/max(awareAudio);

% set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
% plot(awareAudioTS,awareAudioNorm,'g',audioTS,audioNorm,'b');
% legend('Aware audio','Zaurus audio');


%%%%% synchronization %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Take first K mins of audio, normalize, resample to same rate
% Then do a cross-correlation and find the maximal peak. 
% Lowpass filtering seems to help detect the right peak.
K = 5;
A1 = double(audio.values(1:8000*60*K));
A2 = double(awareAudio.values(1:8000*60*K));
A1 = A1 ./ max(A1);
A2 = A2 ./ max(A2);

% do the same for the last K minutes
C1 = double(audio.values(end-8000*60*1.5:end));
C2 = double(awareAudio.values(end-8000*60*1.5:end));
C1 = C1 ./ max(C1);
C2 = C2 ./ max(C2);
audio.values = [];
awareAudio.values = [];

A2b = resample(A2,round(audio.rate),round(awareAudio.rate));
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
subplot(2,1,1); 
plot(A1(1:16:end)); title('First 5 mins of Zaurus audio');
subplot(2,1,2); 
plot(A2(1:16:end)); title('First 5 mins of Aware audio');

B = xcorr(A1,A2b);
fc = 3000; FS = 8000;
[b,a]=butter(4,fc/(FS/2));
B = filter(b,a,B);
B = abs(B-mean(B));
[foo Idx] = max(B);
[sortVals sortIdx] = sort(B,'descend');
fprintf('First five cross-correlation peaks: %d %d %d %d %d\n',sortIdx(1),sortIdx(2),sortIdx(3),sortIdx(4),sortIdx(5));
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
plot(B);
hold on;
for i=1:5
    plot(sortIdx(i),sortVals(i),'go');
    text(sortIdx(i),sortVals(i)*1.02,sprintf('%d',i));
end
offset = sortIdx(1) - length(A1);
fprintf('Offset is %d samples = %3.2f seconds\n',offset,offset/audio.rate);
TSdifference = awareAudio.start - audio.start - offset/audio.rate;
fprintf('Timestamp difference is %3.2f + %3.2f = %3.2f\n',audio.start-awareAudio.start,offset/audio.rate,TSdifference)
awareAudio.start = awareAudio.start - TSdifference;
awareAudio.stop = awareAudio.stop - TSdifference;
awareAudio.timeStamps = awareAudio.timeStamps - TSdifference;
%to do
%awareAcc(:,1) = awareAcc(:,1) - TSdifference;

set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
plot(1:16:length(A1),A1(1:16:end),'b'); title('both audio, shifted, agains index');
hold on;
plot(offset+1:16:offset+length(A2b),A2b(1:16:end),'g');

set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
plot(linspace(audio.start,audio.start+length(A1)/audio.rate,length(A1(1:16:end))),A1(1:16:end),'b'); title('both audio, shifted, against time');
hold on;
plot(linspace(awareAudio.start,awareAudio.start+length(A2)/awareAudio.rate,length(A2(1:16:end))),A2(1:16:end),'g');

% % do the same for the last K minutes
% C1 = double(audio.values(end-8000*60*1.5:end));
% C2 = double(awareAudio.values(end-8000*60*1.5:end));
% C1 = C1 ./ max(C1);
% C2 = C2 ./ max(C2);
%A2 = resample(A2,round(audio.rate),round(awareAudio.rate));
% set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
% subplot(2,1,1); 
% plot(A1(1:16:end)); title('last 5 mins of Zaurus audio');
% subplot(2,1,2); 
% plot(A2(1:16:end)); title('last 5 mins of Aware audio');
% 
% B = xcorr(A1,A2);
% fc = 3000; FS = 8000;
% [b,a]=butter(4,fc/(FS/2));
% B = filter(b,a,B);
% B = abs(B-mean(B));
% [foo Idx] = max(B);
% [sortVals sortIdx] = sort(B,'descend');
% fprintf('First five cross-correlation peaks: %d %d %d %d %d\n',sortIdx(1),sortIdx(2),sortIdx(3),sortIdx(4),sortIdx(5));
% set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
% plot(B);
% hold on;
% for i=1:5
%     plot(sortIdx(i),sortVals(i),'go');
%     text(sortIdx(i),sortVals(i)*1.03,sprintf('%d',i));
% end
% offset = sortIdx(1) - length(A1);
% fprintf('Offset is %d samples = %3.2f seconds\n',offset,offset/audio.rate);
% TSdifference = audio.start-awareAudio.start + offset/audio.rate;
% fprintf('Timestamp difference is %3.2f + %3.2f = %3.2f\n',audio.start-awareAudio.start,offset/audio.rate,TSdifference)
% set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
% plot(1:16:length(A1),A1(1:16:end),'b'); title('both audio, shifted');
% hold on;
% plot(offset+1:16:offset+length(A2),A2(1:16:end),'g');

%for comparison, the clapping at the end should be aligned too
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
plot(linspace(audio.stop-length(C1)/audio.rate,audio.stop,length(C1(1:16:end))),C1(1:16:end),'b'); title('both audio, shifted, against time');
hold on;
plot(linspace(awareAudio.stop-length(C2)/awareAudio.rate,awareAudio.stop,length(C2(1:16:end))),C2(1:16:end),'g');


return

fprintf('Playing back built in audio\n');
wavplay(audioNorm,8000);
fprintf('Playing back AWare audio\n');
wavplay(awareAudioNorm,8000,'async');