%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Written by Mark Blum (mark@media.mit.edu) in August 2005
%%%
%%% This script creates:
%%% a) training/testing data specifically to train each of the 
%%%    four pre-classifiers 
%%% b) a set of testing data, where all four labels are "confident"
%%%    This data set will be the base for post-classification


close all; clear all;
startTime = clock;
saveResults = 1;
DRIVE_LETTER = 'E';
saveNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3\set3_');
plotResults = 0;
LOCATION = 2; SPEECH = 3; POSTURE = 4; EVENT = 5;
%INTERESTING_LABEL_IDX = 0;      % 0 ALL LABELS, 1 timeStamps, 2 location, 3 speech, 4 posture, 5 events, 6 Quality, 7 Interest, 8 Music
%LABEL_CONFIDENCE = [1 1 1 1];   % [location, speech, posture, events] 0 = take all, 1 = only take confident clips

dataLocations{1} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\trainEvent1\trainEvents1_z6000_23_19_23');
dataLocations{2} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\2gmites\2gmites_z6000_00_37_49');
dataLocations{3} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\lunch5\lunch5_z6000_00_25_01');
dataLocations{4} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\workshop1\workshop1_z6000_23_20_36');
dataLocations{5} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\workshop1\afternoon\workshop1b_z6000_00_03_30');
dataLocations{6} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\shopping1\shopping1_z6000_19_30_05');
dataLocations{7} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\evening1\evening1_z6000_19_56_29');
dataLocations{8} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\morning1\morning1_z6000_12_28_08');
dataLocations{9} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\driving1\driving1_z6000_16_33_01');
dataLocations{10} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\lunch2\Mark_z6000_12_22_56');
dataLocations{11} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\mall1\mall1_z6000_12_34_07');
dataLocations{12} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\evening2\evening2_z6000_18_24_06');
dataLocations{13} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\jon1\jon1_z6000_19_43_08');

AUDIO_BUFF_SIZE = [1024 256 256 1024 1024 1024 1024 1024 1024 1024 1024 1024 1024 1024];


%%%%% test if all locations are correct %%%%%%%%%%%%%%%%%%%%%
fprintf('Checking all data locations...');
for i=1:length(dataLocations)
   fid = fopen(strcat(dataLocations{i},'.audio'),'r');
   if (fid == -1)
       fprintf('\nError, could not open file %s\n',strcat(dataLocations{i},'.audio'));
       return;
   end
   fclose(fid);
end
fprintf('done\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Create training clips
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('############################# trainingClips: Location ###################################\n');
trainingClips = [];
for i=2:length(dataLocations);
    if (i==13) continue; end  % don't train location on Jon's data
%     if (i>10) 
%         save('temp','trainingClips');
%         clear trainingClips;
%     end
    rawClips = SignalLoader(dataLocations{i},'aggregate clips',0,'label of interest',LOCATION,...
        'label confidence', [1 0 0 0],'no audio',0,'lower bound',10,'upper bound',540,'normalize','f','plots',0,...
        'seconds prior',0,'seconds after',20,'audio samples per line',AUDIO_BUFF_SIZE(i),'no acc',1);
%     if (i>10) 
%         load('temp');
%     end
    trainingClips = [trainingClips rawClips];
    clear rawClips;
end
[a b c] = findLabels(trainingClips,1,9,0);   
for i=1:length(b)
    trainingClips{b(i)}.label(2) = 5;  %relabel restaurant to "5"
end

if (plotResults), plotClips(trainingClips,'audio'); end
if (saveResults)
    saveName = [saveNamePrefix 'Location'];
    fprintf('Saving training data as %s.mat...',saveName);
    save(saveName,'trainingClips');
end
totalRunTime = etime(clock,startTime);
fprintf('done\nCurrent run time is %5.0f seconds = %2.2f hours\n\n\n',totalRunTime,totalRunTime/60/60);


fprintf('############################# trainingClips: Speech ###################################\n');
clear trainingClips; pack;
trainingClips = [];
for i=1:length(dataLocations);
    if (i==13) continue; end  % don't train speech on Jon's data
    rawClips = SignalLoader(dataLocations{i},'aggregate clips',0,'label of interest',SPEECH,...
        'label confidence', [0 1 0 0],'no audio',0,'lower bound',10,'upper bound',540,'normalize','f','plots',0,...
        'seconds prior',0,'seconds after',20,'audio samples per line',AUDIO_BUFF_SIZE(i),'no acc',1);
    trainingClips = [trainingClips rawClips];
    clear rawClips;
end
%%%%% seperate Clips that we don-t want to train with
[a b c] = findLabels(trainingClips,2,0,0); % cut away the unknown speech class since we don't want to classify it
trainingClips = trainingClips(c);
[a b c] = findLabels(trainingClips,2,6,0);   
speechClips = trainingClips(b);            % use the "speech but unknow" class to see if we recognize speech in general
trainingClips = trainingClips(c);          % cut away the "speech but unknown" class since we don't want to train it

if (plotResults), plotClips(trainingClips,'audio'); end
if (saveResults)
    saveName = [saveNamePrefix 'Speech'];
    fprintf('Saving training data as %s.mat...',saveName);
    save(saveName,'trainingClips','speechClips');
    clear speechClips;
end
totalRunTime = etime(clock,startTime);
fprintf('done\nCurrent run time is %5.0f seconds = %2.2f hours\n\n\n',totalRunTime,totalRunTime/60/60);


fprintf('############################# trainingClips: Posture ###################################\n');
trainingClips = [];
for i=1:length(dataLocations);
    if (i==13) continue; end% don't train on Jon's data
    rawClips = SignalLoader(dataLocations{i},'aggregate clips',1,'label of interest',POSTURE,...
        'label confidence', [0 0 1 0],'no audio',1,'lower bound',10,'upper bound',540,'normalize','f','plots',0,...
        'seconds prior',0,'seconds after',20,'audio samples per line',AUDIO_BUFF_SIZE(i),'no acc',0);
    trainingClips = [trainingClips rawClips];
    clear rawClips;
end
%%%% find clips with problems with the acceleration sensors and remove them
goodClips = [];
for i=1:length(trainingClips)
    if (trainingClips{i}.numAccSamples > 1000)
        goodClips = [goodClips i];
    end
end
trainingClips = trainingClips(goodClips);
if (plotResults), plotClips(trainingClips,'acc1'); end
if (saveResults)
    saveName = [saveNamePrefix 'Posture'];
    fprintf('Saving training data as %s.mat...',saveName);
    save(saveName,'trainingClips');
end
totalRunTime = etime(clock,startTime);
fprintf('done\nCurrent run time is %5.0f seconds = %2.2f hours\n\n\n',totalRunTime,totalRunTime/60/60);


fprintf('############################# trainingClips: Event ###################################\n');
trainingClips = [];
for i=1:length(dataLocations);
    if (i==13) continue; end% don't train on Jon's data
    rawClips = SignalLoader(dataLocations{i},'aggregate clips',1,'label of interest',EVENT,...
        'label confidence', [1 0 0 0],'no audio',1,'lower bound',10,'upper bound',540,'normalize','f','plots',0,...
        'seconds prior',0,'seconds after',20,'audio samples per line',AUDIO_BUFF_SIZE(i),'no acc',0);
    trainingClips = [trainingClips rawClips];
    clear rawClips;
end
%%%% find clips with problems with the acceleration sensors and remove them
goodClips = [];
for i=1:length(trainingClips)
    if (trainingClips{i}.numAccSamples > 1000)
        goodClips = [goodClips i];
    end
end
trainingClips = trainingClips(goodClips);

[a b c] = findLabels(trainingClips,4,8,0); % leave away the manipulation class
trainingClips = trainingClips(c);

if (plotResults), plotClips(trainingClips,'acc1'); end
if (saveResults)
    saveName = [saveNamePrefix 'Event'];
    fprintf('Saving training data as %s.mat...',saveName);
    save(saveName,'trainingClips');
end
clear trainingClips
totalRunTime = etime(clock,startTime);
fprintf('done\nCurrent run time is %5.0f seconds = %2.2f hours\n\n\n',totalRunTime,totalRunTime/60/60);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Create test clips
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('################################ testClips ######################################\n');
testClips = [];

%%% seperate call for this very specific training set
rawClips = SignalLoader(dataLocations{1},'aggregate clips',1,...
    'label of interest',0,'label confidence',[1 1 1 1],'no audio',0,'lower bound',10,...
    'upper bound',540,'normalize','f','seconds prior',0,'seconds after',0,'plots',0,'audio samples per line',1024);
if (plotResults), plotClips(rawClips,'acc1'); end
testClips = [testClips rawClips];
clear rawClips;

%%% these sets are "normal" and share all the options for SignalLoader
for i=2:length(dataLocations);
    if (i==13) continue; end% don't test Jon's data yet
    rawClips = SignalLoader(dataLocations{i},'aggregate clips',0,'label of interest',0,...
        'label confidence', [1 1 1 1],'no audio',0,'lower bound',10,'upper bound',540,'normalize','f','plots',0,...
        'seconds prior',0,'seconds after',20,'audio samples per line',AUDIO_BUFF_SIZE(i),'no acc',0);
    testClips = [testClips rawClips];
    clear rawClips;
end
if (plotResults), plotClips(trainingClips,'acc1'); end
totalRunTime = etime(clock,startTime);
fprintf('done\nCurrent run time is %5.0f seconds = %2.2f hours\n\n\n',totalRunTime,totalRunTime/60/60);


%%%% find clips with problems with the acceleration sensors and remove them
goodClips = [];
for i=1:length(testClips)
    if (testClips{i}.numAccSamples > 1000)
        goodClips = [goodClips i];
    end
end
fprintf('Removing %d clips because of bad acceleration data\n',length(testClips)-length(goodClips));
testClips = testClips(goodClips);


% cut away the "unknown" speech class and "speech but unknown" since we don't want to classify it
[a b c] = findLabels(testClips,2,0,0);   
testClips = testClips(c);
[a b c] = findLabels(testClips,2,6,0);   
testClips = testClips(c); 

% cut away the "unknown posture" class, it is trained by computer generated data
[a b c] = findLabels(testClips,3,0,0);   
testClips = testClips(c);

% cut away the "manipulation event" class, it is trained by computer generated data
[a b c] = findLabels(testClips,4,8,0);   
testClips = testClips(c);

% relabel restaurant location to "5"
[a b c] = findLabels(testClips,1,9,0);   
for i=1:length(b)
    testClips{b(i)}.label(2) = 5;  
end


%%%%% Save test clips %%%%%%%%%%%%%%%%%%%%%%
if (saveResults)
    saveName = [saveNamePrefix 'TestClips'];
    fprintf('Saving testing data as %s.mat...',saveName);
    save(saveName,'testClips');
end


fprintf('\n###################################### done ######################################\n');
totalRunTime = etime(clock,startTime);
fprintf('Total runtime was %5.0f seconds = %2.2f hours\n',totalRunTime,totalRunTime/60/60);
clear all;
% trainAll_testAll;  %this comes next..