%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Written by Mark Blum (mark@media.mit.edu) in September 2005
%%%
%%% In this script several post-classification approaches are 
%%% evaluated. 

format compact;
close all; %clear all; drawnow;
startTime = clock; 
warning off all 
saveResults = 1;
DRIVE_LETTER = 'E';
saveName = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3\set3_uber_classified')';
load(strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3\set3_preclassified_testClips'));


%%%% the format of testClips{i}.classificationXXX
%
%  TS1 p1 p2 ... pn decision label
%  TS2 p1 p2 ... pn decision label
%  ...
%

chainStrings = {'location','speech ','posture','events '};
stateStrings = {{'office','home','outdoors','indoors','restautant','car','street','shop'},...
                {'no speech','me speaking','other speaker','distant voices','loud crowd','laughter'},...
                {'unknown','lie','sit','stand','walk','run','bike'},...
                {'no event','eating','typing','shaking hands','clapping hands','driving','brushing teeth','doing the dishes'}};
observationUNLs = {'/preclass/location','/preclass/speech','/preclass/posture','/preclass/events'};
outputUNLs = {'/class/location','/class/speech','/class/posture','/class/events'};


bnet = mk_influence([8 6 7 8], [8 6 7 8], 'cccc');
for i=1:bnet.nchains
   bnet.B{i} = 0.9*eye(bnet.n_states(i))+0.1/(bnet.n_states(i)-1)*(ones(bnet.n_states(i))-eye(bnet.n_states(i)));
   %bnet.cov{i};
end
for i=1:bnet.nchains
    for j=1:bnet.nchains
        bnet.A{i,j} = mk_stochastic(bnet.A{i,j}.*0+1);
    end
end

bnet.T = mk_stochastic(ones(bnet.nchains));
engine = influence_inf_engine(bnet);
cumIdx = [0 cumsum(bnet.n_states(1:end-1))];


%XML_export_IM('4c_context.xml',bnet,chainStrings,stateStrings,observationUNLs,outputUNLs);
obsLocation = []; obsSpeech = []; obsPosture = []; obsEvents = []; statLabels = [];
previousDataset = ''; timeOffset = 0;
for i=1:length(testClips)
    if (not(strcmp(testClips{i}.dataset,previousDataset)))
        timeOffset = timeOffset + 100000;
    end
    previousDataset = testClips{i}.dataset;
    testClips{i}.classificationLocation(:,1) = testClips{i}.classificationLocation(:,1) + timeOffset;
    testClips{i}.classificationSpeech(:,1) = testClips{i}.classificationSpeech(:,1) + timeOffset;
    testClips{i}.classificationPosture(:,1) = testClips{i}.classificationPosture(:,1) + timeOffset;
    testClips{i}.classificationEvent(:,1) = testClips{i}.classificationEvent(:,1) + timeOffset;
    obsLocation = [obsLocation ; testClips{i}.classificationLocation];
    obsSpeech = [obsSpeech ; testClips{i}.classificationSpeech];
    obsPosture = [obsPosture ; testClips{i}.classificationPosture];
    obsEvents = [obsEvents ; testClips{i}.classificationEvent];
    statLabels = [statLabels ; (testClips{i}.label(2:5)+[0 0 1 1])];
end
observations = {}; timeStamps = {}; labels = {}; decisions = {};
observations{1} = obsLocation(:,2:end-2);
observations{2} = obsSpeech(:,2:end-2);
observations{3} = obsPosture(:,2:end-2);
observations{4} = obsEvents(:,2:end-2);
% startTime = 0;
% endTime = 1000;
% timeStamps{1} = linspace(startTime,endTime,size(obsLocation,1))';
% timeStamps{2} = linspace(startTime,endTime,size(obsSpeech,1))';
% timeStamps{3} = linspace(startTime,endTime,size(obsPosture,1))';
% timeStamps{4} = linspace(startTime,endTime,size(obsEvents,1))';
timeStamps{1} = obsLocation(:,1);
timeStamps{2} = obsSpeech(:,1);
timeStamps{3} = obsPosture(:,1);
timeStamps{4} = obsEvents(:,1);
labels{1} = obsLocation(:,end);
labels{2} = obsSpeech(:,end);
labels{3} = obsPosture(:,end);
labels{4} = obsEvents(:,end);
decisions{1} = obsLocation(:,end-1);
decisions{2} = obsSpeech(:,end-1);
decisions{3} = obsPosture(:,end-1);
decisions{4} = obsEvents(:,end-1);

clear obsLocation obsSpeech obsPosture obsEvents


%%%%% Create event driven Observations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     The idea here is, that we want to perform a classification each time
%     we receive a new observation for any of the chains. The observations
%     for the other chains shall remain at their most recent values.
%     As a result we actually do nchains*numCases classifications
count = 1;
currentIdx = zeros(1,bnet.nchains);
outTime = []; newObservations = []; states = []; newDecisions = [];
while(1)
    nextTime = [];
    newObsAvailable = 0;
    for i=1:bnet.nchains
        if (currentIdx(i) < size(timeStamps{i},1))
            nextTime = [nextTime timeStamps{i}(currentIdx(i)+1)];
            newObsAvailable = 1;
        else
            nextTime = [nextTime NaN];
        end
    end
    if (not(newObsAvailable)) 
        break 
    end
    [outTime(count) newObsIndex] = min(nextTime);
    currentIdx(newObsIndex) = currentIdx(newObsIndex) + 1;
    for i=1:bnet.nchains
        if (currentIdx(i) == 0)
            %newObservations{count,i} = {NaN; ones(bnet.m_outputs(i),1)*NaN };
            newObservations{count,i} = ones(bnet.m_outputs(i),1)*NaN;
            states{count,i} = NaN;
            newDecisions{count,i} = NaN;
            lastIndexWithNaN = count;
        elseif (currentIdx(i) > size(observations{i},1))
           newObservations{count,i} = newObservations{count-1,i};  %repeat most recent values
            newDecisions{count,i} = newDecisions{count-i,i};
        else
            %newObservations{count,i} = {labels{i}(currentIdx(i),:) ; observations{i}(currentIdx(i),:)};
            newObservations{count,i} = observations{i}(currentIdx(i),:)';
            states{count,i} = labels{i}(currentIdx(i));
            newDecisions{count,i} = decisions{i}(currentIdx(i));
        end
    end
    count = count + 1;
end
ev = newObservations(lastIndexWithNaN+1:end,:);
ev_states = cell2mat(states(lastIndexWithNaN+1:end,:));
ev_decisions = cell2mat(newDecisions(lastIndexWithNaN+1:end,:));
numCases = length(ev_states);

totalRunTime = etime(clock,startTime);
fprintf('Case creation time was %5.0f seconds = %2.2f hours\n',totalRunTime,totalRunTime/60/60);

fprintf('Evaluation for sequenced observations:\n');
eval_pre = EvaluatePostClassifications(ev_decisions,ev_states,bnet.n_states,chainStrings,0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Compute certainy measures for pre-classifications
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

certainties = eval_pre.detailedAccuracies;
fp_rate = {};
for c=1:bnet.nchains
    fp_rate{c}=(sum(eval_pre.confusions{c}-diag(diag(eval_pre.confusions{c})),1))./sum(eval_pre.confusions{c},1);
end
certainties = zeros(numCases,bnet.nchains);
for c=1:bnet.nchains
    for t=1:numCases
        ev_weight = 0;
        %caseCertainties(t,c) = (ev{t,c}(ev_decisions(t,c))*ev_weight)+(1-ev_weight)*(1-fp_rate{c}(ev_decisions(t,c))).^;
        %certainties(t,c) = ev{t,c}(ev_decisions(t,c))*((1-fp_rate{c}(ev_decisions(t,c)))-0.75)*4;
        certainties(t,c) = (1-fp_rate{c}(ev_decisions(t,c))).^1;
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Postclassification with Influence Model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('Learning params...\n');
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
bnet_trained = learn_params_influence(engine,ev',100,ev_states');
%%% Fixes for missing data
bnet_trained.B{2}(6,:) = bnet.B{2}(6,:);
bnet_trained.B{3}(1,:) = bnet.B{3}(1,:);
bnet_trained.cov{2}(:,:,5) = bnet.cov{2}(:,:,5)./10000;
bnet_trained.cov{3}(:,:,1) = bnet.cov{3}(:,:,1)./10000;
engine_trained = influence_inf_engine(bnet_trained);
close gcf;
plotInfluences(influence_matrix(bnet_trained),bnet.n_states,stateStrings,'Influence Matrix');

numSlices = 3;
fprintf('Reclassifying data with the learned influence model. numSlices = %d\n',numSlices);
classProbs = cell(1,bnet.nchains); 
mpe_im3 = zeros(numCases,bnet.nchains);
mpe_im3_c = zeros(numCases,bnet.nchains);
for i=1:size(ev,1)
    if mod(i,floor(size(ev,1)/10))==0, fprintf(' %2.0f%%',100/floor(size(ev,1))*i); end
    ev3 = ev((max(i-numSlices+1,1):i),:)';
    path = influence_mpe(engine_trained,ev3);
    mpe_im3(i,:) = path(:,end)';
    for c=1:bnet.nchains
        if (certainties(i,c) < 0.96)
            mpe_im3_c(i,c) = mpe_im3(i,c);
        else
            mpe_im3_c(i,c) = ev_decisions(i,c);
        end
    end
    %classProbs = [classProbs;gamma(end,:)];
end
CHAIN = 0;
fprintf('\nPost-classification with influence model, slow T (bnet_trained) numSlices = %d:\n',numSlices);
eval_im3 = EvaluatePostClassifications(mpe_im3,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with IM numSlices = %d',chainStrings{CHAIN},numSlices)); end
fprintf('\nPost-classification with influence model, slow T (bnet_trained) numSlices = %d, certainty:\n',numSlices);
eval_im3_c = EvaluatePostClassifications(mpe_im3_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with IM numSlices = %d',chainStrings{CHAIN},numSlices)); end
mpe_im = mpe_im3;

numSlices = 7;
fprintf('Reclassifying data with the learned influence model. numSlices = %d\n',numSlices);
classProbs = cell(1,bnet.nchains); 
mpe_im7 = zeros(numCases,bnet.nchains);
for i=1:size(ev,1)
    if mod(i,floor(size(ev,1)/10))==0, fprintf(' %2.0f%%',100/floor(size(ev,1))*i); end
    ev3 = ev((max(i-numSlices+1,1):i),:)';
    path = influence_mpe(engine_trained,ev3);
    mpe_im7(i,:) = path(:,end)';
    %classProbs = [classProbs;gamma(end,:)];
end
fprintf('\nPost-classification with influence model, slow T (bnet_trained) numSlices = %d:\n',numSlices);
eval_im = EvaluatePostClassifications(mpe_im7,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with IM numSlices = %d',chainStrings{CHAIN},numSlices)); end
numSlices = 15;
fprintf('Reclassifying data with the learned influence model. numSlices = %d\n',numSlices);
classProbs = cell(1,bnet.nchains); 
mpe_im15 = zeros(numCases,bnet.nchains);
for i=1:size(ev,1)
    if mod(i,floor(size(ev,1)/10))==0, fprintf(' %2.0f%%',100/floor(size(ev,1))*i); end
    ev3 = ev((max(i-numSlices+1,1):i),:)';
    path = influence_mpe(engine_trained,ev3);
    mpe_im15(i,:) = path(:,end)';
    %classProbs = [classProbs;gamma(end,:)];
end
fprintf('\nPost-classification with influence model, slow T (bnet_trained) numSlices = %d:\n',numSlices);
eval_im = EvaluatePostClassifications(mpe_im15,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with IM numSlices = %d',chainStrings{CHAIN},numSlices)); end
%%%%% influence model with mpe over total dataset %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Reclassifying data with the learned influence model. No slices, mpe over all data\n');
path = influence_mpe(engine_trained,ev');
mpe_im_big = path';
%classProbs = gamma;
fprintf('Post-classification with influence model, slow T (bnet_trained), one big slice:\n');
eval_im_big = EvaluatePostClassifications(mpe_im_big,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with IM one big slice',chainStrings{CHAIN})); end

totalRunTime = etime(clock,startTime);
fprintf('IM time was %5.0f seconds = %2.2f hours\n',totalRunTime,totalRunTime/60/60);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Postclassification with joint probabiliy distribution P(A,B,C,D)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%% Create statistics from sequenced labels (ev_states)
fprintf('computing P(A,B,C,D)...');
MIN_PROB = 0; 
p = ones(bnet.n_states)*MIN_PROB;
for i=1:length(ev_states)
    p(ev_states(i,1),ev_states(i,2),ev_states(i,3),ev_states(i,4)) = p(ev_states(i,1),ev_states(i,2),ev_states(i,3),ev_states(i,4)) + 1;
end
p = p./(length(ev_states)+prod(bnet.n_states)*MIN_PROB); %normalize so the sum over all entries = 1
fprintf('done\nReclassifying data...');

MIN_PROB = 0;
cumIdx = [0 cumsum(bnet.n_states(1:end-1))];
for i=1:bnet.nchains
    cIdx = find((1:bnet.nchains) ~= i);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    caseCount = zeros(bnet.n_states(cIdx(1)),bnet.n_states(cIdx(2)),bnet.n_states(cIdx(3)));
    for s1=1:bnet.n_states(cIdx(1))
        for s2=1:bnet.n_states(cIdx(2))
            for s3=1:bnet.n_states(cIdx(3))
                caseCount(s1,s2,s3) = sum((ev_states(:,cIdx(1)) == s1) & (ev_states(:,cIdx(2)) == s2) & (ev_states(:,cIdx(3)) == s3));
            end
        end
    end
    p_BCD{i} = caseCount + (caseCount.*0)+MIN_PROB;
    p_BCD{i} = p_BCD{i}./(length(ev_states)+prod(bnet.n_states(cIdx))*MIN_PROB); %normalize so the sum over all entries = 1
end

%%% Create statistics from unsequenced labels (statLabels)  (was similar, but less good)
% fprintf('computing P(A,B,C,D)...');
% MIN_PROB = 0; 
% p = ones(bnet.n_states)*MIN_PROB;
% for i=1:length(statLabels)
%     p(statLabels(i,1),statLabels(i,2),statLabels(i,3),statLabels(i,4)) = p(statLabels(i,1),statLabels(i,2),statLabels(i,3),statLabels(i,4)) + 1;
% end
% p = p./(length(statLabels)+prod(bnet.n_states)*MIN_PROB); %normalize so the sum over all entries = 1
% fprintf('done\nReclassifying data...');
% 
% MIN_PROB = 0;
% cumIdx = [0 cumsum(bnet.n_states(1:end-1))];
% for i=1:bnet.nchains
%     cIdx = find((1:bnet.nchains) ~= i);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
%     caseCount = zeros(bnet.n_states(cIdx(1)),bnet.n_states(cIdx(2)),bnet.n_states(cIdx(3)));
%     for s1=1:bnet.n_states(cIdx(1))
%         for s2=1:bnet.n_states(cIdx(2))
%             for s3=1:bnet.n_states(cIdx(3))
%                 caseCount(s1,s2,s3) = sum((statLabels(:,cIdx(1)) == s1) & (statLabels(:,cIdx(2)) == s2) & (statLabels(:,cIdx(3)) == s3));
%             end
%         end
%     end
%     p_BCD{i} = caseCount + (caseCount.*0)+MIN_PROB;
%     p_BCD{i} = p_BCD{i}./(length(statLabels)+prod(bnet.n_states(cIdx))*MIN_PROB); %normalize so the sum over all entries = 1
% end

classProbs = cell(1,bnet.nchains); P_log = zeros(1,numCases*31); count = 0;
newCertainties = zeros(numCases,bnet.nchains);
mpe_jp_es = zeros(numCases,bnet.nchains);
mpe_jp_c_es = zeros(numCases,bnet.nchains);
mpe_jp_priors_es = zeros(numCases,bnet.nchains);
for c=1:bnet.nchains
    priors_jp{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c}= zeros(numCases,bnet.n_states(c));
    for t=1:numCases;
        if (c==1) p_denominator = p_BCD{1}(ev_states(t,2),ev_states(t,3),ev_states(t,4)); end
        if (c==2) p_denominator = p_BCD{2}(ev_states(t,1),ev_states(t,3),ev_states(t,4)); end
        if (c==3) p_denominator = p_BCD{3}(ev_states(t,1),ev_states(t,2),ev_states(t,4)); end
        if (c==4) p_denominator = p_BCD{4}(ev_states(t,1),ev_states(t,2),ev_states(t,3)); end
        if (p_denominator==0) fprintf('0'); end
        for s=1:bnet.n_states(c)
            if (c==1) P = p(s,ev_states(t,2),ev_states(t,3),ev_states(t,4)) / p_denominator; end
            if (c==2) P = p(ev_states(t,1),s,ev_states(t,3),ev_states(t,4)) / p_denominator; end
            if (c==3) P = p(ev_states(t,1),ev_states(t,2),s,ev_states(t,4)) / p_denominator; end
            if (c==4) P = p(ev_states(t,1),ev_states(t,2),ev_states(t,3),s) / p_denominator; end
            classProbs{c}(t,s) = P*ev{t,c}(s);
            count = count+1;
            P_log(count) = P;
            priors_jp{c}(t,s) = P;
        end
        [foo mpe_jp_es(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_jp_es(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp_c_es(t,c) = mpe_jp_es(t,c);
        else
            mpe_jp_c_es(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp_priors_es(t,c)] = max(priors_jp{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with common PDFs (ev_states used):\n');
eval_jp = EvaluatePostClassifications(mpe_jp_es,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A,B,C,D) (ev_states used)',chainStrings{CHAIN})); end
fprintf('done\nEvaluation after reclassification with common PDFs (ev_states used) (with certainty):\n');
eval_jp_c = EvaluatePostClassifications(mpe_jp_c_es,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A,B,C,D) (ev_states used), with certainty',chainStrings{CHAIN})); end
fprintf('done\nEvaluation after using common PDF priors only (ev_states used):\n');
eval_jp_priors = EvaluatePostClassifications(mpe_jp_priors_es,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A,B,C,D) (ev_states used), priors only',chainStrings{CHAIN})); end


%%%% if ev_decisions are used instead of ev_states, then we need MINPROB > 0, 
%%%% so the probabilities are never equal 0 (0/0 = NAN --> wrong results)

fprintf('computing P(A,B,C,D)...');
MIN_PROB = 1; 
p = ones(bnet.n_states)*MIN_PROB;
for i=1:length(ev_states)
    p(ev_states(i,1),ev_states(i,2),ev_states(i,3),ev_states(i,4)) = p(ev_states(i,1),ev_states(i,2),ev_states(i,3),ev_states(i,4)) + 1;
end
p = p./(length(ev_states)+prod(bnet.n_states)*MIN_PROB); %normalize so the sum over all entries = 1
fprintf('done\nReclassifying data...');

MIN_PROB = 1;
cumIdx = [0 cumsum(bnet.n_states(1:end-1))];
for i=1:bnet.nchains
    cIdx = find((1:bnet.nchains) ~= i);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    caseCount = zeros(bnet.n_states(cIdx(1)),bnet.n_states(cIdx(2)),bnet.n_states(cIdx(3)));
    for s1=1:bnet.n_states(cIdx(1))
        for s2=1:bnet.n_states(cIdx(2))
            for s3=1:bnet.n_states(cIdx(3))
                caseCount(s1,s2,s3) = sum((ev_states(:,cIdx(1)) == s1) & (ev_states(:,cIdx(2)) == s2) & (ev_states(:,cIdx(3)) == s3));
            end
        end
    end
    p_BCD{i} = caseCount + (caseCount.*0)+MIN_PROB;
    p_BCD{i} = p_BCD{i}./(length(ev_states)+prod(bnet.n_states(cIdx))*MIN_PROB); %normalize so the sum over all entries = 1
end

classProbs = cell(1,bnet.nchains); P_log = zeros(1,numCases*31); count = 0;
newCertainties = zeros(numCases,bnet.nchains);
mpe_jp = zeros(numCases,bnet.nchains);
mpe_jp_c = zeros(numCases,bnet.nchains);
mpe_jp_priors = zeros(numCases,bnet.nchains);
for c=1:bnet.nchains
    priors_jp{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c}= zeros(numCases,bnet.n_states(c));
    for t=1:numCases;
        if (c==1) p_denominator = p_BCD{1}(ev_decisions(t,2),ev_decisions(t,3),ev_decisions(t,4)); end
        if (c==2) p_denominator = p_BCD{2}(ev_decisions(t,1),ev_decisions(t,3),ev_decisions(t,4)); end
        if (c==3) p_denominator = p_BCD{3}(ev_decisions(t,1),ev_decisions(t,2),ev_decisions(t,4)); end
        if (c==4) p_denominator = p_BCD{4}(ev_decisions(t,1),ev_decisions(t,2),ev_decisions(t,3)); end
        for s=1:bnet.n_states(c)
            if (c==1) P = p(s,ev_decisions(t,2),ev_decisions(t,3),ev_decisions(t,4)) / p_denominator; end
            if (c==2) P = p(ev_decisions(t,1),s,ev_decisions(t,3),ev_decisions(t,4)) / p_denominator; end
            if (c==3) P = p(ev_decisions(t,1),ev_decisions(t,2),s,ev_decisions(t,4)) / p_denominator; end
            if (c==4) P = p(ev_decisions(t,1),ev_decisions(t,2),ev_decisions(t,3),s) / p_denominator; end
            classProbs{c}(t,s) = P*ev{t,c}(s);
            count = count+1;
            P_log(count) = P;
            priors_jp{c}(t,s) = P;
        end
        [foo mpe_jp(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_jp(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp_c(t,c) = mpe_jp(t,c);
        else
            mpe_jp_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp_priors(t,c)] = max(priors_jp{c}(t,:));
    end
end

CHAIN = 0;
fprintf('done\nEvaluation after reclassification with common PDFs:\n');
eval_jp = EvaluatePostClassifications(mpe_jp,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A,B,C,D)',chainStrings{CHAIN})); end
fprintf('done\nEvaluation after reclassification with common PDFs (with certainty):\n');
eval_jp_c = EvaluatePostClassifications(mpe_jp_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A,B,C,D), with certainty',chainStrings{CHAIN})); end
fprintf('done\nEvaluation after using common PDF priors only:\n');
eval_jp_priors = EvaluatePostClassifications(mpe_jp_priors,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A,B,C,D) (ev_states used), priors only',chainStrings{CHAIN})); end


%%%%% P(A,B,C,D) with location labels %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
classProbs = cell(1,bnet.nchains); %P_log = zeros(1,numCases*31); count = 0;
newCertainties = zeros(numCases,bnet.nchains);
mpe_jp_loc = zeros(numCases,bnet.nchains);
mpe_jp_loc_c = zeros(numCases,bnet.nchains);
mpe_jp_loc_priors = zeros(numCases,bnet.nchains);
for c=1:bnet.nchains
    priors_jp{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c}= zeros(numCases,bnet.n_states(c));
    for t=1:numCases;
        if (c==1) p_denominator = p_BCD{1}(ev_decisions(t,2),ev_decisions(t,3),ev_decisions(t,4)); end
        if (c==2) p_denominator = p_BCD{2}(ev_states(t,1),ev_decisions(t,3),ev_decisions(t,4)); end
        if (c==3) p_denominator = p_BCD{3}(ev_states(t,1),ev_decisions(t,2),ev_decisions(t,4)); end
        if (c==4) p_denominator = p_BCD{4}(ev_states(t,1),ev_decisions(t,2),ev_decisions(t,3)); end
        for s=1:bnet.n_states(c)
            if (c==1) P = p(s,ev_decisions(t,2),ev_decisions(t,3),ev_decisions(t,4)) / p_denominator; end
            if (c==2) P = p(ev_states(t,1),s,ev_decisions(t,3),ev_decisions(t,4)) / p_denominator; end
            if (c==3) P = p(ev_states(t,1),ev_decisions(t,2),s,ev_decisions(t,4)) / p_denominator; end
            if (c==4) P = p(ev_states(t,1),ev_decisions(t,2),ev_decisions(t,3),s) / p_denominator; end
            classProbs{c}(t,s) = P*ev{t,c}(s);
            %count = count+1;
            %P_log(count) = P;
            priors_jp{c}(t,s) = P;
        end
        [foo mpe_jp_loc(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_jp_loc(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp_loc_c(t,c) = mpe_jp_loc(t,c);
        else
            mpe_jp_loc_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp_loc_priors(t,c)] = max(priors_jp{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with common PDFs, using location labels:\n');
eval_jp_loc = EvaluatePostClassifications(mpe_jp_loc,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A,B,C,D), using location labels',chainStrings{CHAIN})); end
fprintf('done\nEvaluation after reclassification with common PDFs, using location labels (with certainty):\n');
eval_jp_loc_c = EvaluatePostClassifications(mpe_jp_loc_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A,B,C,D), with certainty, using location labels',chainStrings{CHAIN})); end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Postclassification with pairwise conditional proabilites P(A|B) etc.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% compute P(A|B) P(A|C) P(A|D) P(B|A) P(A|C) P(A|D) etc
fprintf('computing P(A|B), P(A|C) etc ...');
caseCount = [];
cumIdx = [0 cumsum(bnet.n_states(1:end-1))];
for c1=1:bnet.nchains
    for s1=1:bnet.n_states(c1)
        for c2=1:bnet.nchains
            for s2=1:bnet.n_states(c2)
                caseCount(cumIdx(c1)+s1,cumIdx(c2)+s2) = sum((ev_states(:,c1) == s1) & (ev_states(:,c2) == s2));
            end
        end
    end
end
MIN_PROB = 0;
p_AB = caseCount + ones(sum(bnet.n_states))*MIN_PROB;
p_AB = p_AB./(numCases+(MIN_PROB*sum(bnet.n_states)^2)/16); %normalize so the sum over all entries = 1
plotInfluences(p_AB,bnet.n_states,stateStrings,'P(A,B)');
p_A = diag(p_AB)';
%%% P(A|B) = P(A,B)/P(B)
for i=1:length(p_A)
    p_AgB_old(i,:) = p_AB(i,:)./p_A;
end
plotInfluences(p_AgB_old',bnet.n_states,stateStrings,'pairwise conditional probabilities P(A|B)');

%%% the version using statLabels is correcter. ev_states contains "faulty"
%%% 4-tuples that introduce some P(x|y) > 0 where it should actually be 0

fprintf('done\ncomputing P(A|B), P(A|C) version statLabels ...');
caseCount = [];
cumIdx = [0 cumsum(bnet.n_states(1:end-1))];
for c1=1:bnet.nchains
    for s1=1:bnet.n_states(c1)
        for c2=1:bnet.nchains
            for s2=1:bnet.n_states(c2)
                caseCount(cumIdx(c1)+s1,cumIdx(c2)+s2) = sum((statLabels(:,c1) == s1) & (statLabels(:,c2) == s2));
            end
        end
    end
end
MIN_PROB = 0;
p_AB = caseCount + ones(sum(bnet.n_states))*MIN_PROB;
p_AB = p_AB./(numCases+(MIN_PROB*sum(bnet.n_states)^2)/16); %normalize so the sum over all entries = 1
plotInfluences(p_AB,bnet.n_states,stateStrings,'P(A,B)');
p_A = diag(p_AB)';
% P(A|B) = P(A,B)/P(B)
for i=1:length(p_A)
    p_AgB_new(i,:) = p_AB(i,:)./p_A;
end

p_AgB = p_AgB_new;
plotInfluences(p_AgB_new',bnet.n_states,stateStrings,'pairwise conditional probabilities P(A|B)');


%%% use only P(A) 
fprintf('done\nReclassifying data P(A)...');
classProbs = cell(1,bnet.nchains); 
mpe_jp2_pA = zeros(numCases,bnet.nchains);
mpe_jp2_pA_c = zeros(numCases,bnet.nchains);
for c=1:bnet.nchains
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            classProbs{c}(t,s) = p_A(cumIdx(c)+s)*ev{t,c}(s);
        end
        [foo mpe_jp2_pA(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_jp2_pA(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp2_pA_c(t,c) = mpe_jp2_pA(t,c);
        else
            mpe_jp2_pA_c(t,c) = ev_decisions(t,c);
        end
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(A|B):\n');
eval_jp2_pA = EvaluatePostClassifications(mpe_jp2_pA,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A|B)',chainStrings{CHAIN}));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(A|B), with certainty:\n');
eval_jp2_pA_c = EvaluatePostClassifications(mpe_jp2_pA_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A|B), with certainty',chainStrings{CHAIN}));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end

%%% Convex combination 
fprintf('done\nReclassifying data, P(A|B), convex combination, using ev_states...');
classProbs = cell(1,bnet.nchains);
mpe_jp2_evs = zeros(numCases,bnet.nchains);
mpe_jp2_evs_c = zeros(numCases,bnet.nchains);
mpe_jp2_evs_priors = zeros(numCases,bnet.nchains);
for c=1:bnet.nchains
    priors_jp2_evs{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_decisions(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+ev_decisions(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+ev_decisions(t,cIdx(3)));
            priors_jp2_evs{c}(t,s) = (P1/3+P2/3+P3/3);
            classProbs{c}(t,s) = priors_jp2_evs{c}(t,s)*ev{t,c}(s);
        end
        [foo mpe_jp2_evs(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_jp2_evs(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp2_evs_c(t,c) = mpe_jp2_evs(t,c);
        else
            mpe_jp2_evs_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp2_evs_priors(t,c)] = max(priors_jp2_evs{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(A|B)  using ev_states:\n');
eval_jp2_evs = EvaluatePostClassifications(mpe_jp2_evs,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A|B)',chainStrings{CHAIN}));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(A|B), with certainty:\n');
eval_jp2_evs_c = EvaluatePostClassifications(mpe_jp2_evs_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A|B), with certainty',chainStrings{CHAIN}));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end

fprintf('done\nReclassifying data, P(A|B), convex combination,...');
classProbs = cell(1,bnet.nchains); 
mpe_jp2 = zeros(numCases,bnet.nchains);
mpe_jp2_c = zeros(numCases,bnet.nchains);
mpe_jp2_priors = zeros(numCases,bnet.nchains);
for c=1:bnet.nchains
    priors_jp2{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_decisions(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+ev_decisions(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+ev_decisions(t,cIdx(3)));
            priors_jp2{c}(t,s) = (P1/3+P2/3+P3/3);
            %priors_jp2{c}(t,s) = (P1*P2*P3);  % this was a bad idea
            classProbs{c}(t,s) = priors_jp2{c}(t,s)*ev{t,c}(s);
        end
        [foo mpe_jp2(t,c)] = max(classProbs{c}(t,:)); 
        if (foo==0), mpe_jp2(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp2_c(t,c) = mpe_jp2(t,c);
        else
            mpe_jp2_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp2_priors(t,c)] = max(priors_jp2{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(A|B):\n');
eval_jp2 = EvaluatePostClassifications(mpe_jp2,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A|B)',chainStrings{CHAIN}));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(A|B), with certainty:\n');
eval_jp2_c = EvaluatePostClassifications(mpe_jp2_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A|B), with certainty',chainStrings{CHAIN}));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end

fprintf('done\nReclassifying data, P(A|B), convex combination, location labels...');
classProbs = cell(1,bnet.nchains);
mpe_jp2_loc = zeros(numCases,bnet.nchains);
mpe_jp2_loc_priors = zeros(numCases,bnet.nchains);
for c=1:bnet.nchains
    priors_jp2_loc{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            if (cIdx(1) == 1)
                P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_states(t,cIdx(1)));
            else
                P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_decisions(t,cIdx(1)));
            end
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+ev_decisions(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+ev_decisions(t,cIdx(3)));
            priors_jp2_loc{c}(t,s) = (P1/3+P2/3+P3/3);
            classProbs{c}(t,s) = priors_jp2_loc{c}(t,s)*ev{t,c}(s);
        end
        [foo mpe_jp2_loc(t,c)] = max(classProbs{c}(t,:)); 
        if (foo==0), mpe_jp2_loc(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp2_loc_c(t,c) = mpe_jp2_loc(t,c);
        else
            mpe_jp2_loc_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp2_loc_priors(t,c)] = max(priors_jp2_loc{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(A|B), convext combination, using location labels:\n');
eval_jp2_loc = EvaluatePostClassifications(mpe_jp2_loc,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A|B), using location labels',chainStrings{CHAIN}));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(A|B), convext combination, using location labels, with certainty:\n');
eval_jp2_loc_c = EvaluatePostClassifications(mpe_jp2_loc_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A|B), using location labels, with certainty',chainStrings{CHAIN}));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end


%%%%% P(X|Y)^(1/K) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('Reclassifying data, P(X|Y)^(1/K) no location labels, using ev_states...');
classProbs = cell(1,bnet.nchains); 
mpe_jp3_noLoc_evs = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_evs_c = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_evs_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3_noLoc_evs{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_states(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+ev_states(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+ev_states(t,cIdx(3)));
            q = 1/K;
            priors_jp3_noLoc_evs{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs{c}(t,s) = priors_jp3_noLoc_evs{c}(t,s)*ev{t,c}(s);
            %P_log(s) = (P1/3+P2/3+P3/3);
        end
        %count = count+1;
        %priors_jp3_noLoc_evs(t,c) = P_log(mpe_jp3_noLoc_evs(t,c));
        [foo mpe_jp3_noLoc_evs(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_jp3_noLoc_evs(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs{c}(t,ev_decisions(t,c)) / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
        %if (certaintyOfPreClassification(t,c) < certainties(t,c) & certainties(t,c) < 0.96)
        %if (certaintyOfPreClassification(t,c) < certainties(t,c) & certainties(t,c) < 0.96 & newCertainties(t,c) > certainties(t,c))
            mpe_jp3_noLoc_evs_c(t,c) = mpe_jp3_noLoc_evs(t,c);
            classProbs{c}(t,:) = normalize(classProbs{c}(t,:));
        else
            mpe_jp3_noLoc_evs_c(t,c) = ev_decisions(t,c);
            classProbs{c}(t,:) = ev{t,c}';
        end
        [foo mpe_jp3_noLoc_evs_priors(t,c)] = max(priors_jp3_noLoc_evs{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) no location labels, using ev_states!:\n',K);
eval_jp3_noLoc_evs = EvaluatePostClassifications(mpe_jp3_noLoc_evs,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), with certainty:\n',K);
eval_jp3_noLoc_evs_c = EvaluatePostClassifications(mpe_jp3_noLoc_evs_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end

fprintf('Reclassifying data, P(X|Y)^(1/K) no location labels...');
classProbs = cell(1,bnet.nchains); 
mpe_jp3_noLoc = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_c = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3_noLoc{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_decisions(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+ev_decisions(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+ev_decisions(t,cIdx(3)));
            q = 1/K;
            priors_jp3_noLoc{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            priors_jp3_noLoc{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs{c}(t,s) = priors_jp3_noLoc{c}(t,s)*ev{t,c}(s);
            %P_log(s) = (P1/3+P2/3+P3/3);
        end
        %count = count+1;
        %priors_jp3_noLoc(t,c) = P_log(mpe_jp3_noLoc(t,c));
        [foo mpe_jp3_noLoc(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_jp3_noLoc(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs{c}(t,ev_decisions(t,c)) / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
        %if (certaintyOfPreClassification(t,c) < certainties(t,c) & certainties(t,c) < 0.96)
        %if (certaintyOfPreClassification(t,c) < certainties(t,c) & certainties(t,c) < 0.96 & newCertainties(t,c) > certainties(t,c))
            mpe_jp3_noLoc_c(t,c) = mpe_jp3_noLoc(t,c);
            classProbs{c}(t,:) = normalize(classProbs{c}(t,:));
        else
            mpe_jp3_noLoc_c(t,c) = ev_decisions(t,c);
            classProbs{c}(t,:) = ev{t,c}';
        end
        [foo mpe_jp3_noLoc_priors(t,c)] = max(priors_jp3_noLoc{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) no location labels:\n',K);
eval_jp3_noLoc = EvaluatePostClassifications(mpe_jp3_noLoc,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), with certainty:\n',K);
eval_jp3_noLoc_c = EvaluatePostClassifications(mpe_jp3_noLoc_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end

%%%%% P(X|Y)^(1/K) after P(X|Y)^(1/K) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Reclassifying data, P(X|Y)^(1/K) after P(X|Y)^(1/K) ...');
classProbs2 = cell(1,bnet.nchains); 
mpe_jp3_noLoc_re = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_re_c = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_re_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3_noLoc{c} = zeros(numCases,bnet.n_states(c));
    classProbs2{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+mpe_jp3_noLoc_c(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+mpe_jp3_noLoc_c(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+mpe_jp3_noLoc_c(t,cIdx(3)));
            q = 1/K;
            priors_jp3_noLoc{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs2{c}(t,s) = priors_jp3_noLoc{c}(t,s)*ev{t,c}(s);%classProbs{c}(t,s);
        end
        [foo mpe_jp3_noLoc_re(t,c)] = max(classProbs2{c}(t,:));
        if (foo==0), mpe_jp3_noLoc_re(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs2{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs2{c}(t,ev_decisions(t,c)) / sum(classProbs2{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp3_noLoc_re_c(t,c) = mpe_jp3_noLoc_re(t,c);
        else
            mpe_jp3_noLoc_re_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp3_noLoc_re_priors(t,c)] = max(priors_jp3_noLoc{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) TWICE, NO location :\n',K);
eval_jp3_noLoc_re = EvaluatePostClassifications(mpe_jp3_noLoc_re,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), TWICE, with certainty:\n',K);
eval_jp3_noLoc_c_re = EvaluatePostClassifications(mpe_jp3_noLoc_re_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end

%%%%% P(X|Y)^(1/K) after P(X|Y)^(1/K) again %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Reclassifying data, P(X|Y)^(1/K) after P(X|Y)^(1/K) again ...');
classProbs3 = cell(1,bnet.nchains); %rior2_jp2 = zeros(numCases,bnet.nchains); count = 0;
mpe_jp3_noLoc_re2 = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_re2_c = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_re2_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3_noLoc{c} = zeros(numCases,bnet.n_states(c));
    classProbs3{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+mpe_jp3_noLoc_re_c(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+mpe_jp3_noLoc_re_c(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+mpe_jp3_noLoc_re_c(t,cIdx(3)));
            q = 1/K;
            priors_jp3_noLoc{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs3{c}(t,s) = priors_jp3_noLoc{c}(t,s)*ev{t,c}(s);%classProbs{c}(t,s);
        end
        [foo mpe_jp3_noLoc_re2(t,c)] = max(classProbs3{c}(t,:));
        if (foo==0), mpe_jp3_noLoc_re2(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs3{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs3{c}(t,ev_decisions(t,c)) / sum(classProbs3{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp3_noLoc_re2_c(t,c) = mpe_jp3_noLoc_re2(t,c);
        else
            mpe_jp3_noLoc_re2_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp3_noLoc_re2_priors(t,c)] = max(priors_jp3_noLoc{c}(t,:));
    end
end
CHAIN = 3;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) 3 TIMES, NO location:\n',K);
eval_jp3_noLoc_re2 = EvaluatePostClassifications(mpe_jp3_noLoc_re2,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), 3 TIMES, with certainty:\n',K);
eval_jp3_noLoc_c_re2 = EvaluatePostClassifications(mpe_jp3_noLoc_re2_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end

%%%%% P(X|Y)^(1/10) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Reclassifying data, P(X|Y)^(1/K), location labels...');
classProbs = cell(1,bnet.nchains); %rior2_jp2 = zeros(numCases,bnet.nchains); count = 0;
mpe_jp3 = zeros(numCases,bnet.nchains);
mpe_jp3_c = zeros(numCases,bnet.nchains);
mpe_jp3_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            if (cIdx(1) == 1)
                P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_states(t,cIdx(1)));
            else
                P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_decisions(t,cIdx(1)));
            end
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+ev_decisions(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+ev_decisions(t,cIdx(3)));
            q = 1/K;
            priors_jp3{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs{c}(t,s) = priors_jp3{c}(t,s)*ev{t,c}(s);
        end
        [foo mpe_jp3(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_jp3(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs{c}(t,ev_decisions(t,c)) / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp3_c(t,c) = mpe_jp3(t,c);
            classProbs{c}(t,:) = normalize(classProbs{c}(t,:));
        else
            mpe_jp3_c(t,c) = ev_decisions(t,c);
            classProbs{c}(t,:) = ev{t,c}';
        end
        [foo mpe_jp3_priors(t,c)] = max(priors_jp3{c}(t,:));
    end
end
CHAIN = 3;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), location labels:\n',K);
eval_jp3 = EvaluatePostClassifications(mpe_jp3,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), location labels, with certainty:\n',K);
eval_jp3_c = EvaluatePostClassifications(mpe_jp3_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end

fprintf('Reclassifying data, P(X|Y)^(1/K) after P(X|Y)^(1/K) location labels...');
classProbs2 = cell(1,bnet.nchains); %rior2_jp2 = zeros(numCases,bnet.nchains); count = 0;
mpe_jp3_re = zeros(numCases,bnet.nchains);
mpe_jp3_re_c = zeros(numCases,bnet.nchains);
mpe_jp3_re_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3{c} = zeros(numCases,bnet.n_states(c));
    classProbs2{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            if (cIdx(1) == 1)
                P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_states(t,cIdx(1)));
            else
                P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+mpe_jp3_c(t,cIdx(1)));
            end
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+mpe_jp3_c(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+mpe_jp3_c(t,cIdx(3)));
            q = 1/K;
            priors_jp3{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs2{c}(t,s) = priors_jp3{c}(t,s)*ev{t,c}(s);%classProbs{c}(t,s);
        end
        [foo mpe_jp3_re(t,c)] = max(classProbs2{c}(t,:));
        if (foo==0), mpe_jp3_re(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs2{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs2{c}(t,ev_decisions(t,c)) / sum(classProbs2{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp3_re_c(t,c) = mpe_jp3_re(t,c);
        else
            mpe_jp3_re_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp3_re_priors(t,c)] = max(priors_jp3{c}(t,:));
    end
end
CHAIN = 3;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) TWICE, location labels:\n',K);
eval_jp3_re = EvaluatePostClassifications(mpe_jp3_re,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), TWICE, location labels with certainty:\n',K);
eval_jp3_c_re = EvaluatePostClassifications(mpe_jp3_re_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end

fprintf('Reclassifying data, P(X|Y)^(1/K) after P(X|Y)^(1/K) location labels again ...');
classProbs3 = cell(1,bnet.nchains); %rior2_jp2 = zeros(numCases,bnet.nchains); count = 0;
mpe_jp3_re2 = zeros(numCases,bnet.nchains);
mpe_jp3_re2_c = zeros(numCases,bnet.nchains);
mpe_jp3_re2_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3{c} = zeros(numCases,bnet.n_states(c));
    classProbs3{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            if (cIdx(1) == 1)
                P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_states(t,cIdx(1)));
            else
                P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+mpe_jp3_re_c(t,cIdx(1)));
            end
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+mpe_jp3_re_c(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+mpe_jp3_re_c(t,cIdx(3)));
            q = 1/K;
            priors_jp3{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs3{c}(t,s) = priors_jp3{c}(t,s)*ev{t,c}(s);%classProbs{c}(t,s);
        end
        [foo mpe_jp3_re2(t,c)] = max(classProbs3{c}(t,:));
        newCertainties(t,c) = foo / sum(classProbs3{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs3{c}(t,ev_decisions(t,c)) / sum(classProbs3{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp3_re2_c(t,c) = mpe_jp3_re2(t,c);
        else
            mpe_jp3_re2_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp3_re2_priors(t,c)] = max(priors_jp3{c}(t,:));
    end
end
CHAIN = 3;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) 3 TIMES, location labels :\n',K);
eval_jp3_re2 = EvaluatePostClassifications(mpe_jp3_re2,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), 3 TIMES, location labels with certainty:\n',K);
eval_jp3_c_re2 = EvaluatePostClassifications(mpe_jp3_re2_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Some more combinations, but not much interesting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%% P(X|Y)^(1/K) after influence matrix %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Reclassifying data, P(X|Y)^(1/K) after IM...');
classProbs = cell(1,bnet.nchains); %rior2_jp2 = zeros(numCases,bnet.nchains); count = 0;
mpe_im_jp3 = zeros(numCases,bnet.nchains);
mpe_im_jp3_c = zeros(numCases,bnet.nchains);
mpe_im_jp3_priors = zeros(numCases,bnet.nchains);
K = 5;
for c=1:bnet.nchains
    priors_im_jp3{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+mpe_im(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+mpe_im(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+mpe_im(t,cIdx(3)));
            q = 1/K;
            priors_im_jp3{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs{c}(t,s) = priors_im_jp3{c}(t,s)*ev{t,c}(s);
            %P_log(s) = (P1/3+P2/3+P3/3);
        end
        %count = count+1;
        %priors_im_jp3(t,c) = P_log(mpe_im_jp3(t,c));
        [foo mpe_im_jp3(t,c)] = max(classProbs{c}(t,:)); 
        if (foo==0), mpe_im_jp3(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_im_jp3_c(t,c) = mpe_im_jp3(t,c);
        else
            mpe_im_jp3_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_im_jp3_priors(t,c)] = max(priors_im_jp3{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) after IM:\n',K);
eval_im_jp3 = EvaluatePostClassifications(mpe_im_jp3,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d) after IM',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) after IM, with certainty:\n',K);
eval_im_jp3_c = EvaluatePostClassifications(mpe_im_jp3_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d) after IM, with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end

%%%%% P(X|Y)^(1/8) after influence matrix %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Reclassifying data, P(X|Y)^(1/K) after IM, location labels...');
classProbs = cell(1,bnet.nchains); %rior2_jp2 = zeros(numCases,bnet.nchains); count = 0;
mpe_im_jp3_Loc = zeros(numCases,bnet.nchains);
mpe_im_jp3_Loc_c = zeros(numCases,bnet.nchains);
mpe_im_jp3_Loc_priors = zeros(numCases,bnet.nchains);
K = 5;
for c=1:bnet.nchains
    priors_im_jp3_Loc{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            if (cIdx(1) == 1)
                P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_states(t,cIdx(1)));
            else
                P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+mpe_im(t,cIdx(1)));
            end
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+mpe_im(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+mpe_im(t,cIdx(3)));
            q = 1/K;
            priors_im_jp3_Loc{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs{c}(t,s) = priors_im_jp3_Loc{c}(t,s)*ev{t,c}(s);
            %P_log(s) = (P1/3+P2/3+P3/3);
        end
        %count = count+1;
        %priors_im_jp3_Loc(t,c) = P_log(mpe_im_jp3_Loc(t,c));
        [foo mpe_im_jp3_Loc(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_im_jp3_Loc(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_im_jp3_Loc_c(t,c) = mpe_im_jp3_Loc(t,c);
        else
            mpe_im_jp3_Loc_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_im_jp3_Loc_priors(t,c)] = max(priors_im_jp3_Loc{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) after IM, location labels:\n',K);
eval_im_jp3_loc = EvaluatePostClassifications(mpe_im_jp3_Loc,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d) after IM',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) after IM, location labels, with certainty:\n',K);
eval_im_jp3_c_loc = EvaluatePostClassifications(mpe_im_jp3_Loc_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d) after IM, with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end


%%%%% convex combination after influence matrix %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Reclassifying data, convex combination after IM...');
classProbs = cell(1,bnet.nchains); %rior2_jp2 = zeros(numCases,bnet.nchains); count = 0;
mpe_im_jp = zeros(numCases,bnet.nchains);
mpe_im_jp_priors = zeros(numCases,bnet.nchains);
for c=1:bnet.nchains
    priors_im_jp{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+mpe_im(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+mpe_im(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+mpe_im(t,cIdx(3)));
            priors_im_jp{c}(t,s) = (P1/3+P2/3+P3/3);
            classProbs{c}(t,s) = priors_im_jp{c}(t,s)*ev{t,c}(s);
            %P_log(s) = (P1/3+P2/3+P3/3);
        end
        %count = count+1;
        %priors_im_jp(t,c) = P_log(mpe_im_jp(t,c));
        [foo mpe_im_jp(t,c)] = max(classProbs{c}(t,:)); 
        if (foo==0), mpe_im_jp(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_im_jp_c(t,c) = mpe_im_jp(t,c);
        else
            mpe_im_jp_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_im_jp_priors(t,c)] = max(priors_im_jp{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with IM and P(A|B):\n');
eval_im_jp = EvaluatePostClassifications(mpe_im_jp,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with IM and P(A|B), MIN-PROB = %2.3f',chainStrings{CHAIN},MIN_PROB));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with IM and P(A|B), with certainty:\n');
eval_im_jp_c = EvaluatePostClassifications(mpe_im_jp_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(A|B), with certainty, MIN-PROB = %2.3f',chainStrings{CHAIN},MIN_PROB));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end



%%%%% influence matrix with replaced bnet.priors %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

numSlices = 15;
engine_trained_rp = engine_trained;
% for c=1:bnet.nchains
%     engine_trained_rp.bnet.Pi{c} = p_A(cumIdx(c)+1:cumIdx(c)+bnet.n_states(c));
% end
engine_trained_rp.bnet.Pi{2} = [1 0 0 0 0 0];
engine_trained_rp.bnet.Pi{3} = [0 0 1 0 0 0 0];
engine_trained_rp.bnet.Pi{4} = [1 0 0 0 0 0 0 0];
fprintf('Reclassifying data with the IM, replaced priors. numSlices = %d\n',numSlices);
classProbs = cell(1,bnet.nchains); 
mpe_im_rp = zeros(numCases,bnet.nchains);
for i=1:size(ev,1)
    if mod(i,floor(size(ev,1)/10))==0, fprintf(' %2.0f%%',100/floor(size(ev,1))*i); end
    ev3 = ev((max(i-numSlices+1,1):i),:)';
    path = influence_mpe(engine_trained_rp,ev3);
    mpe_im_rp(i,:) = path(:,end)';
    %classProbs = [classProbs;gamma(end,:)];
end
CHAIN = 0;
fprintf('Post-classification with influence model, slow T, replaced priors (engine2):\n');
eval_im_rp = EvaluatePostClassifications(mpe_im_rp,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with IM and replaced priors',chainStrings{CHAIN})); end


%%% testing different weights for K

fprintf('Reclassifying data, P(X|Y)^(1/K) ADAPTED no location labels...');
classProbs = cell(1,bnet.nchains); %rior2_jp2 = zeros(numCases,bnet.nchains); count = 0;
mpe_jp3_noLoc = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_c = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3_noLoc{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_decisions(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+ev_decisions(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+ev_decisions(t,cIdx(3)));
            if (cIdx(1) == 1) priors_jp3_noLoc{c}(t,s) = (P1^(1/4))*(P2^(1/1))*(P3^(1/4)); end
            if (cIdx(1) == 2) priors_jp3_noLoc{c}(t,s) = (P1^(1/7))*(P2^(1/1))*(P3^(1/4)); end
            if (cIdx(1) == 3) priors_jp3_noLoc{c}(t,s) = (P1^(1/7))*(P2^(1/4))*(P3^(1/4)); end
            if (cIdx(1) == 4) priors_jp3_noLoc{c}(t,s) = (P1^(1/7))*(P2^(1/4))*(P3^(1/1)); end
            %priors_jp3_noLoc{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs{c}(t,s) = priors_jp3_noLoc{c}(t,s)*ev{t,c}(s);
            %P_log(s) = (P1/3+P2/3+P3/3);
        end
        %count = count+1;
        %priors_jp3_noLoc(t,c) = P_log(mpe_jp3_noLoc(t,c));
        [foo mpe_jp3_noLoc(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_jp3_noLoc(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs{c}(t,ev_decisions(t,c)) / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
        %if (certaintyOfPreClassification(t,c) < certainties(t,c) & certainties(t,c) < 0.96)
        %if (certaintyOfPreClassification(t,c) < certainties(t,c) & certainties(t,c) < 0.96 & newCertainties(t,c) > certainties(t,c))
            mpe_jp3_noLoc_c(t,c) = mpe_jp3_noLoc(t,c);
            classProbs{c}(t,:) = normalize(classProbs{c}(t,:));
        else
            mpe_jp3_noLoc_c(t,c) = ev_decisions(t,c);
            classProbs{c}(t,:) = ev{t,c}';
        end
        [foo mpe_jp3_noLoc_priors(t,c)] = max(priors_jp3_noLoc{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) ADAPTED no location labels:\n',K);
eval_jp3_noLoc = EvaluatePostClassifications(mpe_jp3_noLoc,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), with certainty:\n',K);
eval_jp3_noLoc_c = EvaluatePostClassifications(mpe_jp3_noLoc_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end


save(strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\verify\models'),'p_AgB','p','p_BCD','bnet_trained','engine_trained','certainties');

save(saveName);
totalRunTime = etime(clock,startTime);
fprintf('Total runtime was %5.0f seconds = %2.2f hours\n',totalRunTime,totalRunTime/60/60);
