%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Written by Mark Blum (mark@media.mit.edu) in September 2005
%%%
%%% Note: this file does not completly compile, it only contains
%%% the suggested post-classificatin scheme

format compact;
close all; %clear all; drawnow;
startTime = clock; 
warning off all 
saveResults = 1;
DRIVE_LETTER = 'E';
saveName = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3\set3_uber_classified')';
load(strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3\set3_preclassified_testClips'));


%%%% the format of testClips{i}.classificationXXX
%
%  TS1 p1 p2 ... pn decision label
%  TS2 p1 p2 ... pn decision label
%  ...
%

chainStrings = {'location','speech ','posture','events '};
stateStrings = {{'office','home','outdoors','indoors','restautant','car','street','shop'},...
                {'no speech','me speaking','other speaker','distant voices','loud crowd','laughter'},...
                {'unknown','lie','sit','stand','walk','run','bike'},...
                {'no event','eating','typing','shaking hands','clapping hands','driving','brushing teeth','doing the dishes'}};
observationUNLs = {'/preclass/location','/preclass/speech','/preclass/posture','/preclass/events'};
outputUNLs = {'/class/location','/class/speech','/class/posture','/class/events'};


cumIdx = [0 cumsum(bnet.n_states(1:end-1))];

obsLocation = []; obsSpeech = []; obsPosture = []; obsEvents = []; statLabels = [];
previousDataset = ''; timeOffset = 0;
for i=1:length(testClips)
    if (not(strcmp(testClips{i}.dataset,previousDataset)))
        timeOffset = timeOffset + 100000;
    end
    previousDataset = testClips{i}.dataset;
    testClips{i}.classificationLocation(:,1) = testClips{i}.classificationLocation(:,1) + timeOffset;
    testClips{i}.classificationSpeech(:,1) = testClips{i}.classificationSpeech(:,1) + timeOffset;
    testClips{i}.classificationPosture(:,1) = testClips{i}.classificationPosture(:,1) + timeOffset;
    testClips{i}.classificationEvent(:,1) = testClips{i}.classificationEvent(:,1) + timeOffset;
    obsLocation = [obsLocation ; testClips{i}.classificationLocation];
    obsSpeech = [obsSpeech ; testClips{i}.classificationSpeech];
    obsPosture = [obsPosture ; testClips{i}.classificationPosture];
    obsEvents = [obsEvents ; testClips{i}.classificationEvent];
    statLabels = [statLabels ; (testClips{i}.label(2:5)+[0 0 1 1])];
end
observations = {}; timeStamps = {}; labels = {}; decisions = {};
observations{1} = obsLocation(:,2:end-2);
observations{2} = obsSpeech(:,2:end-2);
observations{3} = obsPosture(:,2:end-2);
observations{4} = obsEvents(:,2:end-2);
timeStamps{1} = obsLocation(:,1);
timeStamps{2} = obsSpeech(:,1);
timeStamps{3} = obsPosture(:,1);
timeStamps{4} = obsEvents(:,1);
labels{1} = obsLocation(:,end);
labels{2} = obsSpeech(:,end);
labels{3} = obsPosture(:,end);
labels{4} = obsEvents(:,end);
decisions{1} = obsLocation(:,end-1);
decisions{2} = obsSpeech(:,end-1);
decisions{3} = obsPosture(:,end-1);
decisions{4} = obsEvents(:,end-1);

clear obsLocation obsSpeech obsPosture obsEvents


%%%%% Create event driven Observations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     The idea here is, that we want to perform a classification each time
%     we receive a new observation for any of the chains. The observations
%     for the other chains shall remain at their most recent values.
%     As a result we actually do nchains*numCases classifications
count = 1;
currentIdx = zeros(1,bnet.nchains);
outTime = []; newObservations = []; states = []; newDecisions = [];
while(1)
    nextTime = [];
    newObsAvailable = 0;
    for i=1:bnet.nchains
        if (currentIdx(i) < size(timeStamps{i},1))
            nextTime = [nextTime timeStamps{i}(currentIdx(i)+1)];
            newObsAvailable = 1;
        else
            nextTime = [nextTime NaN];
        end
    end
    if (not(newObsAvailable)) 
        break 
    end
    [outTime(count) newObsIndex] = min(nextTime);
    currentIdx(newObsIndex) = currentIdx(newObsIndex) + 1;
    for i=1:bnet.nchains
        if (currentIdx(i) == 0)
            %newObservations{count,i} = {NaN; ones(bnet.m_outputs(i),1)*NaN };
            newObservations{count,i} = ones(bnet.m_outputs(i),1)*NaN;
            states{count,i} = NaN;
            newDecisions{count,i} = NaN;
            lastIndexWithNaN = count;
        elseif (currentIdx(i) > size(observations{i},1))
           newObservations{count,i} = newObservations{count-1,i};  %repeat most recent values
            newDecisions{count,i} = newDecisions{count-i,i};
        else
            %newObservations{count,i} = {labels{i}(currentIdx(i),:) ; observations{i}(currentIdx(i),:)};
            newObservations{count,i} = observations{i}(currentIdx(i),:)';
            states{count,i} = labels{i}(currentIdx(i));
            newDecisions{count,i} = decisions{i}(currentIdx(i));
        end
    end
    count = count + 1;
end
ev = newObservations(lastIndexWithNaN+1:end,:);
ev_states = cell2mat(states(lastIndexWithNaN+1:end,:));
ev_decisions = cell2mat(newDecisions(lastIndexWithNaN+1:end,:));
numCases = length(ev_states);

fprintf('Evaluation for sequenced observations:\n');
eval_pre = EvaluatePostClassifications(ev_decisions,ev_states,bnet.n_states,chainStrings,0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Compute certainy measures for pre-classifications
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

certainties = eval_pre.detailedAccuracies;
fp_rate = {};
for c=1:bnet.nchains
    fp_rate{c}=(sum(eval_pre.confusions{c}-diag(diag(eval_pre.confusions{c})),1))./sum(eval_pre.confusions{c},1);
end
certainties = zeros(numCases,bnet.nchains);
for c=1:bnet.nchains
    for t=1:numCases
        ev_weight = 0;
        %caseCertainties(t,c) = (ev{t,c}(ev_decisions(t,c))*ev_weight)+(1-ev_weight)*(1-fp_rate{c}(ev_decisions(t,c))).^;
        %certainties(t,c) = ev{t,c}(ev_decisions(t,c))*((1-fp_rate{c}(ev_decisions(t,c)))-0.75)*4;
        certainties(t,c) = (1-fp_rate{c}(ev_decisions(t,c))).^1;
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Postclassification with pairwise conditional proabilites P(A|B) etc.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('done\ncomputing P(A|B), P(A|C) version statLabels ...');
caseCount = [];
cumIdx = [0 cumsum(bnet.n_states(1:end-1))];
for c1=1:bnet.nchains
    for s1=1:bnet.n_states(c1)
        for c2=1:bnet.nchains
            for s2=1:bnet.n_states(c2)
                caseCount(cumIdx(c1)+s1,cumIdx(c2)+s2) = sum((statLabels(:,c1) == s1) & (statLabels(:,c2) == s2));
            end
        end
    end
end
MIN_PROB = 0;
p_AB = caseCount + ones(sum(bnet.n_states))*MIN_PROB;
p_AB = p_AB./(numCases+(MIN_PROB*sum(bnet.n_states)^2)/16); %normalize so the sum over all entries = 1
plotInfluences(p_AB,bnet.n_states,stateStrings,'P(A,B)');
p_A = diag(p_AB)';
% P(A|B) = P(A,B)/P(B)
for i=1:length(p_A)
    p_AgB_new(i,:) = p_AB(i,:)./p_A;
end

p_AgB = p_AgB_new;
plotInfluences(p_AgB_new',bnet.n_states,stateStrings,'pairwise conditional probabilities P(A|B)');


fprintf('Reclassifying data, P(X|Y)^(1/K) no location labels...');
classProbs = cell(1,bnet.nchains); 
mpe_jp3_noLoc = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_c = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3_noLoc{c} = zeros(numCases,bnet.n_states(c));
    classProbs{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+ev_decisions(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+ev_decisions(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+ev_decisions(t,cIdx(3)));
            q = 1/K;
            priors_jp3_noLoc{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            priors_jp3_noLoc{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs{c}(t,s) = priors_jp3_noLoc{c}(t,s)*ev{t,c}(s);
            %P_log(s) = (P1/3+P2/3+P3/3);
        end
        %count = count+1;
        %priors_jp3_noLoc(t,c) = P_log(mpe_jp3_noLoc(t,c));
        [foo mpe_jp3_noLoc(t,c)] = max(classProbs{c}(t,:));
        if (foo==0), mpe_jp3_noLoc(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs{c}(t,ev_decisions(t,c)) / sum(classProbs{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
        %if (certaintyOfPreClassification(t,c) < certainties(t,c) & certainties(t,c) < 0.96)
        %if (certaintyOfPreClassification(t,c) < certainties(t,c) & certainties(t,c) < 0.96 & newCertainties(t,c) > certainties(t,c))
            mpe_jp3_noLoc_c(t,c) = mpe_jp3_noLoc(t,c);
            classProbs{c}(t,:) = normalize(classProbs{c}(t,:));
        else
            mpe_jp3_noLoc_c(t,c) = ev_decisions(t,c);
            classProbs{c}(t,:) = ev{t,c}';
        end
        [foo mpe_jp3_noLoc_priors(t,c)] = max(priors_jp3_noLoc{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) no location labels:\n',K);
eval_jp3_noLoc = EvaluatePostClassifications(mpe_jp3_noLoc,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), with certainty:\n',K);
eval_jp3_noLoc_c = EvaluatePostClassifications(mpe_jp3_noLoc_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end

%%%%% P(X|Y)^(1/K) after P(X|Y)^(1/K) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Reclassifying data, P(X|Y)^(1/K) after P(X|Y)^(1/K) ...');
classProbs2 = cell(1,bnet.nchains); 
mpe_jp3_noLoc_re = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_re_c = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_re_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3_noLoc{c} = zeros(numCases,bnet.n_states(c));
    classProbs2{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+mpe_jp3_noLoc_c(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+mpe_jp3_noLoc_c(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+mpe_jp3_noLoc_c(t,cIdx(3)));
            q = 1/K;
            priors_jp3_noLoc{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs2{c}(t,s) = priors_jp3_noLoc{c}(t,s)*ev{t,c}(s);%classProbs{c}(t,s);
        end
        [foo mpe_jp3_noLoc_re(t,c)] = max(classProbs2{c}(t,:));
        if (foo==0), mpe_jp3_noLoc_re(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs2{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs2{c}(t,ev_decisions(t,c)) / sum(classProbs2{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp3_noLoc_re_c(t,c) = mpe_jp3_noLoc_re(t,c);
        else
            mpe_jp3_noLoc_re_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp3_noLoc_re_priors(t,c)] = max(priors_jp3_noLoc{c}(t,:));
    end
end
CHAIN = 0;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) TWICE, NO location :\n',K);
eval_jp3_noLoc_re = EvaluatePostClassifications(mpe_jp3_noLoc_re,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), TWICE, with certainty:\n',K);
eval_jp3_noLoc_c_re = EvaluatePostClassifications(mpe_jp3_noLoc_re_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end

%%%%% P(X|Y)^(1/K) after P(X|Y)^(1/K) again %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Reclassifying data, P(X|Y)^(1/K) after P(X|Y)^(1/K) again ...');
classProbs3 = cell(1,bnet.nchains); %rior2_jp2 = zeros(numCases,bnet.nchains); count = 0;
mpe_jp3_noLoc_re2 = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_re2_c = zeros(numCases,bnet.nchains);
mpe_jp3_noLoc_re2_priors = zeros(numCases,bnet.nchains);
newCertainties = zeros(numCases,bnet.nchains);
certaintyOfPreClassification = zeros(numCases,bnet.nchains);
K = 4;
for c=1:bnet.nchains
    priors_jp3_noLoc{c} = zeros(numCases,bnet.n_states(c));
    classProbs3{c} = zeros(numCases,bnet.n_states(c));
    cIdx = find((1:bnet.nchains) ~= c);  %cIdx = [2 3 4] , [1 3 4], [1 2 4] , [1 2 3]
    for t=1:numCases
        for s=1:bnet.n_states(c)
            P1 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(1))+mpe_jp3_noLoc_re_c(t,cIdx(1)));
            P2 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(2))+mpe_jp3_noLoc_re_c(t,cIdx(2)));
            P3 = p_AgB(cumIdx(c)+s,cumIdx(cIdx(3))+mpe_jp3_noLoc_re_c(t,cIdx(3)));
            q = 1/K;
            priors_jp3_noLoc{c}(t,s) = (P1^q)*(P2^q)*(P3^q);
            classProbs3{c}(t,s) = priors_jp3_noLoc{c}(t,s)*ev{t,c}(s);%classProbs{c}(t,s);
        end
        [foo mpe_jp3_noLoc_re2(t,c)] = max(classProbs3{c}(t,:));
        if (foo==0), mpe_jp3_noLoc_re2(t,c) = ev_decisions(t,c); end
        newCertainties(t,c) = foo / sum(classProbs3{c}(t,:));
        certaintyOfPreClassification(t,c) = classProbs3{c}(t,ev_decisions(t,c)) / sum(classProbs3{c}(t,:));
        if (newCertainties(t,c) > certainties(t,c) & certainties(t,c) < 0.96)
            mpe_jp3_noLoc_re2_c(t,c) = mpe_jp3_noLoc_re2(t,c);
        else
            mpe_jp3_noLoc_re2_c(t,c) = ev_decisions(t,c);
        end
        [foo mpe_jp3_noLoc_re2_priors(t,c)] = max(priors_jp3_noLoc{c}(t,:));
    end
end
CHAIN = 3;
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d) 3 TIMES, NO location:\n',K);
eval_jp3_noLoc_re2 = EvaluatePostClassifications(mpe_jp3_noLoc_re2,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d)',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); end
fprintf('done\nEvaluation after reclassification with P(X|Y)^(1/%d), 3 TIMES, with certainty:\n',K);
eval_jp3_noLoc_c_re2 = EvaluatePostClassifications(mpe_jp3_noLoc_re2_c,ev_states,bnet.n_states,chainStrings,CHAIN,ev_decisions,stateStrings);
if (CHAIN) title(sprintf('%s: with P(X|Y)^(1/%d), with certainty',chainStrings{CHAIN},K));
hold on; plot(certainties(:,CHAIN),'b'); plot(newCertainties(:,CHAIN),'r'); plot(certaintyOfPreClassification(:,CHAIN),'g');end


save(strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\verify\models'),'p_AgB','p','p_BCD','bnet_trained','engine_trained','certainties');

save(saveName);
totalRunTime = etime(clock,startTime);
fprintf('Total runtime was %5.0f seconds = %2.2f hours\n',totalRunTime,totalRunTime/60/60);
