%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Written by Mark Blum (mark@media.mit.edu) in September 2005
%%%
%%% Applies previously learned Gaussian classifiers to previously 
%%% loaded clips.

format compact;
startTime = clock; tic;
saveResults = 1;
DRIVE_LETTER = 'E';
loadNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\verify\verify_');
CloadNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3_ttr_1\set3_preclassified_');
saveNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\verify\verify_preclassified_');

load([CloadNamePrefix 'C']); % load previously trained Classifiers C.Posture, C.Event, etc

load([loadNamePrefix 'Posture']);

%%%%% Posture Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
all_instances = findLabels(testClips, 3)
fprintf('################################### Posture ###################################\n');
fprintf('Computing features for test clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.accFeaturesPosture = AccelFeature(testClips{i},C.Posture.featureType,C.Posture.WINDOWSIZE,C.Posture.TIMESTEP,C.Posture.NUM_FEATURES);
end
fprintf('Classifying test clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.classificationPosture = [GClassifier(testClips{i}.accFeaturesPosture,C.Posture.bnet)...
        ones(size(testClips{i}.accFeaturesPosture,1),1)*(testClips{i}.label(C.Posture.INTERESTING_LABEL_IDX)+1)];
end
fprintf('\n');
[C.Posture.accuracy C.Posture.confusion] = EvaluateClassifications(testClips,'posture',C.Posture.LABEL_STRINGS);
if (saveResults)
    saveName = [saveNamePrefix 'Posture'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'testClips','C');
end
fprintf('done!\n\n');
clear testClips

%%%% the format of classificationXXX
%
%  TS1 p1 p2 ... pn decision label
%  TS2 p1 p2 ... pn decision label
%  ...
%
load([loadNamePrefix 'Event']);

%%%%% Event Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
all_instances = findLabels(testClips, 4)
fprintf('################################### Event ###################################\n');
fprintf('Computing features for test clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.accFeaturesEvent = AccelFeature(testClips{i},C.Event.featureType,C.Event.WINDOWSIZE,C.Event.TIMESTEP,C.Event.NUM_FEATURES);
end
fprintf('Classifying test clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.classificationEvent = [GClassifier(testClips{i}.accFeaturesEvent,C.Event.bnet)...
        ones(size(testClips{i}.accFeaturesEvent,1),1)*(testClips{i}.label(C.Event.INTERESTING_LABEL_IDX)+1)];
end
fprintf('\n');
[C.Event.accuracy C.Event.confusion] = EvaluateClassifications(testClips,'event',C.Event.LABEL_STRINGS);
if (saveResults)
    saveName = [saveNamePrefix 'Event'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'testClips','C');
end
fprintf('done!\n\n');
clear testClips

NORMALIZATION = [1 8.540838e-001 1.451333e+001 4.480513e+000 3.008648e-002 4.305218e+002 1.336457e-001 1.157824e+002 3.351075e-001 6.731011e-003 2.713941e+004];

%%%%% Location Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load([loadNamePrefix 'Location']);

instances = findLabels(testClips, 1)
fprintf('################################### Location ###################################\n');
fprintf('Computing features for test clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    audio = double(testClips{i}.audio)/2^15;
    numFrames = floor( (length(audio) - (C.Location.FRAMESIZE-C.Location.FRAMESTEP)) / C.Location.FRAMESTEP );
    featureTimestamps = linspace(testClips{i}.audioStart,testClips{i}.audioStop,numFrames)';
    features_temp = speech_features(audio,C.Location.FRAMESIZE,C.Location.FRAMESTEP,C.Location.rms/5)';
    testClips{i}.audioFeatures = [featureTimestamps features_temp(:,[4 6 3 7 1])]; % same order of features as in c-code
    testClips{i}.audioFeaturesLocation = AccelFeature(testClips{i},'audio_mean_var',C.Location.WINDOWSIZE,C.Location.TIMESTEP,C.Location.NUM_FEATURES);
    testClips{i}.audioFeaturesLocation = testClips{i}.audioFeaturesLocation ./ (ones(size(testClips{i}.audioFeaturesLocation,1),1)*NORMALIZATION);
    testClips{i}.audioFeatures = [];  % free memory
end
fprintf('\n'); clear audio clip;

fprintf('Classifying clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.classificationLocation = [GClassifier(testClips{i}.audioFeaturesLocation,C.Location.bnet)...
        ones(size(testClips{i}.audioFeaturesLocation,1),1)*(testClips{i}.label(C.Location.INTERESTING_LABEL_IDX))+1];
end
fprintf('\n');
[C.Location.accuracy C.Location.confusion] = EvaluateClassifications(testClips,'location',C.Location.LABEL_STRINGS);
if (saveResults)
    saveName = [saveNamePrefix 'Location'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'testClips','C');
end
fprintf('done!\n\n');


%%%%% Speech Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load([loadNamePrefix 'Speech']);

instances = findLabels(testClips, 2)
fprintf('################################### Speech ###################################\n');

fprintf('Computing features for test clips');
for i=1:length(testClips) 
    fprintf(' %d',i);
    audio = double(testClips{i}.audio)/2^15;
    numFrames = floor( (length(audio) - (C.Speech.FRAMESIZE-C.Speech.FRAMESTEP)) / C.Speech.FRAMESTEP );
    featureTimestamps = linspace(testClips{i}.audioStart,testClips{i}.audioStop,numFrames)';
    features_temp = speech_features(audio,C.Speech.FRAMESIZE,C.Speech.FRAMESTEP,C.Speech.rms/5)';
    testClips{i}.audioFeatures = [featureTimestamps features_temp(:,[4 6 3 7 1])]; % same order of features as in c-code
    testClips{i}.audioFeaturesSpeech = AccelFeature(testClips{i},'audio_mean_var',C.Speech.WINDOWSIZE,C.Speech.TIMESTEP,C.Speech.NUM_FEATURES);
    testClips{i}.audioFeaturesSpeech = testClips{i}.audioFeaturesSpeech ./ (ones(size(testClips{i}.audioFeaturesSpeech,1),1)*NORMALIZATION);
    testClips{i}.audioFeatures = [];  % free memory
end
fprintf('\n'); clear audio clip;

fprintf('Classifying clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.classificationSpeech = [GClassifier(testClips{i}.audioFeaturesSpeech,C.Speech.bnet)...
        ones(size(testClips{i}.audioFeaturesSpeech,1),1)*(testClips{i}.label(C.Speech.INTERESTING_LABEL_IDX))];
end
fprintf('\n');
[C.Speech.accuracy C.Speech.confusion] = EvaluateClassifications(testClips,'speech',C.Speech.LABEL_STRINGS);
speakingConfusion = [C.Speech.confusion(1,1) sum(C.Speech.confusion(1,2:end)); sum(C.Speech.confusion(2:end,1)) sum(sum(C.Speech.confusion(2:end,2:end)))]
if (saveResults)
    saveName = [saveNamePrefix 'Speech'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'testClips','C');
end
fprintf('done!\n\n');
clear testClips

totalRunTime = etime(clock,startTime);
fprintf('Total runtime was %5.0f seconds = %2.2f hours\n',totalRunTime,totalRunTime/60/60);