%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Written by Mark Blum (mark@media.mit.edu) in September 2005
%%%
%%% This script loads cross-validation datasets with SignalLoader.
%%% The data is classified according to a previously trained model.
%%% The results are evaluated.
%%% 

close all; clear all; drawnow;
startTime = clock;
saveResults = 1;
DRIVE_LETTER = 'E';
saveNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\verify\verify_235_');
plotResults = 0;
LOCATION = 2; SPEECH = 3; POSTURE = 4; EVENT = 5;

dataLocations{1} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\verify5\verify5_z6000_21_57_44');
dataLocations{2} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\verify5\verify5b\name_z6000_00_01_24');
dataLocations{3} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\verify2\verify2_z6000_12_57_34');
dataLocations{4} = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\verify3\verify3_z6000_17_33_14');

AUDIO_BUFF_SIZE = [1024 1024 1024 1024];

%%%%% test if all locations are correct %%%%%%%%%%%%%%%%%%%%%
fprintf('Checking all data locations...');
for i=1:length(dataLocations)
   fid = fopen(strcat(dataLocations{i},'.audio'),'r');
   if (fid == -1)
       fprintf('\nError, could not open file %s\n',strcat(dataLocations{i},'.audio'));
       return;
   end
   fclose(fid);
end
fprintf('done\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Create verification clips
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf('############################# trainingClips: Location ###################################\n');
testClips = [];
for i=1:length(dataLocations);
    rawClips = SignalLoader(dataLocations{i},'aggregate clips',0,'label of interest',LOCATION,...
        'label confidence', [1 0 0 0],'no audio',0,'lower bound',10,'upper bound',540,'normalize','f','plots',0,...
        'seconds prior',0,'seconds after',20,'audio samples per line',AUDIO_BUFF_SIZE(i),'no acc',1);
    testClips = [testClips rawClips];
    clear rawClips;
end
[a b c] = findLabels(testClips,1,9,0);   
for i=1:length(b)
    testClips{b(i)}.label(2) = 5;  %relabel restaurant to "5"
end

if (plotResults), plotClips(testClips,'audio'); end
if (saveResults)
    saveName = [saveNamePrefix 'Location'];
    fprintf('Saving training data as %s.mat...',saveName);
    save(saveName,'testClips');
end
totalRunTime = etime(clock,startTime);
fprintf('done\nCurrent run time is %5.0f seconds = %2.2f hours\n\n\n',totalRunTime,totalRunTime/60/60);


fprintf('############################# testClips: Speech ###################################\n');
testClips = [];
for i=1:length(dataLocations);
    rawClips = SignalLoader(dataLocations{i},'aggregate clips',0,'label of interest',SPEECH,...
        'label confidence', [0 1 0 0],'no audio',0,'lower bound',10,'upper bound',540,'normalize','f','plots',0,...
        'seconds prior',0,'seconds after',20,'audio samples per line',AUDIO_BUFF_SIZE(i),'no acc',1);
    testClips = [testClips rawClips];
    clear rawClips;
end
%%%%% seperate Clips that we don-t want to train with
[a b c] = findLabels(testClips,2,0,0); % cut away the unknown speech class since we don't want to classify it
testClips = testClips(c);
[a b c] = findLabels(testClips,2,6,0);   
speechClips = testClips(b);            % use the "speech but unknow" class to see if we recognize speech in general
testClips = testClips(c);          % cut away the "speech but unknown" class since we don't want to train it

if (plotResults), plotClips(testClips,'audio'); end
if (saveResults)
    saveName = [saveNamePrefix 'Speech'];
    fprintf('Saving training data as %s.mat...',saveName);
    save(saveName,'testClips','speechClips');
    clear speechClips;
end
totalRunTime = etime(clock,startTime);
fprintf('done\nCurrent run time is %5.0f seconds = %2.2f hours\n\n\n',totalRunTime,totalRunTime/60/60);


fprintf('############################# testClips: Posture ###################################\n');
testClips = [];
for i=1:length(dataLocations);
    if (i==13) continue; end% don't train on Jon's data
    rawClips = SignalLoader(dataLocations{i},'aggregate clips',1,'label of interest',POSTURE,...
        'label confidence', [0 0 1 0],'no audio',1,'lower bound',10,'upper bound',540,'normalize','f','plots',0,...
        'seconds prior',0,'seconds after',20,'audio samples per line',AUDIO_BUFF_SIZE(i),'no acc',0,'oneG_correction',1);
    testClips = [testClips rawClips];
    clear rawClips;
end
%%%% find clips with problems with the acceleration sensors and remove them
goodClips = [];
for i=1:length(testClips)
    if (testClips{i}.numAccSamples > 1000)
        goodClips = [goodClips i];
    end
end
testClips = testClips(goodClips);
if (plotResults), plotClips(testClips,'acc1'); end
if (saveResults)
    saveName = [saveNamePrefix 'Posture'];
    fprintf('Saving training data as %s.mat...',saveName);
    save(saveName,'testClips');
end
totalRunTime = etime(clock,startTime);
fprintf('done\nCurrent run time is %5.0f seconds = %2.2f hours\n\n\n',totalRunTime,totalRunTime/60/60);


fprintf('############################# testClips: Event ###################################\n');
testClips = [];
for i=1:length(dataLocations);
    if (i==13) continue; end% don't train on Jon's data
    rawClips = SignalLoader(dataLocations{i},'aggregate clips',1,'label of interest',EVENT,...
        'label confidence', [1 0 0 0],'no audio',1,'lower bound',10,'upper bound',540,'normalize','f','plots',0,...
        'seconds prior',0,'seconds after',20,'audio samples per line',AUDIO_BUFF_SIZE(i),'no acc',0,'oneG_correction',1);
    testClips = [testClips rawClips];
    clear rawClips;
end
%%%% find clips with problems with the acceleration sensors and remove them
goodClips = [];
for i=1:length(testClips)
    if (testClips{i}.numAccSamples > 1000)
        goodClips = [goodClips i];
    end
end
testClips = testClips(goodClips);
[a b c] = findLabels(testClips,4,8,0); % leave away the manipulation class
testClips = testClips(c);
if (plotResults), plotClips(testClips,'acc1'); end
if (saveResults)
    saveName = [saveNamePrefix 'Event'];
    fprintf('Saving training data as %s.mat...',saveName);
    save(saveName,'testClips');
end
clear testClips
totalRunTime = etime(clock,startTime);
fprintf('done\nCurrent run time is %5.0f seconds = %2.2f hours\n\n\n',totalRunTime,totalRunTime/60/60);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Classify verification clips
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


format compact;
%startTime = clock; tic;
saveResults = 1;
DRIVE_LETTER = 'E';
loadNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\verify\verify_235_');
CloadNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3_ttr_1\set3_preclassified_');
CloadNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3_ttr_1\set3_preclassified_');
saveNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\verify\verify_235_preclassified_');

load([CloadNamePrefix 'C']); % load previously trained Classifiers C.Posture, C.Event, etc

load([loadNamePrefix 'Posture']);

%%%%% Posture Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
all_instances = findLabels(testClips, 3)
fprintf('################################### Posture ###################################\n');
fprintf('Computing features for test clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.accFeaturesPosture = AccelFeature(testClips{i},C.Posture.featureType,C.Posture.WINDOWSIZE,C.Posture.TIMESTEP,C.Posture.NUM_FEATURES);
end
fprintf('Classifying test clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.classificationPosture = [GClassifier(testClips{i}.accFeaturesPosture,C.Posture.bnet)...
        ones(size(testClips{i}.accFeaturesPosture,1),1)*(testClips{i}.label(C.Posture.INTERESTING_LABEL_IDX)+1)];
end
fprintf('\n');
[C.Posture.accuracy C.Posture.confusion] = EvaluateClassifications(testClips,'posture',C.Posture.LABEL_STRINGS);
if (saveResults)
    saveName = [saveNamePrefix 'Posture'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'testClips','C');
end
fprintf('done!\n\n');
clear testClips

%%%% the format of classificationXXX
%
%  TS1 p1 p2 ... pn decision label
%  TS2 p1 p2 ... pn decision label
%  ...
%
load([loadNamePrefix 'Event']);

%%%%% Event Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
all_instances = findLabels(testClips, 4)
fprintf('################################### Event ###################################\n');
fprintf('Computing features for test clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.accFeaturesEvent = AccelFeature(testClips{i},C.Event.featureType,C.Event.WINDOWSIZE,C.Event.TIMESTEP,C.Event.NUM_FEATURES);
end
fprintf('Classifying test clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.classificationEvent = [GClassifier(testClips{i}.accFeaturesEvent,C.Event.bnet)...
        ones(size(testClips{i}.accFeaturesEvent,1),1)*(testClips{i}.label(C.Event.INTERESTING_LABEL_IDX)+1)];
end
fprintf('\n');
[C.Event.accuracy C.Event.confusion] = EvaluateClassifications(testClips,'event',C.Event.LABEL_STRINGS);
if (saveResults)
    saveName = [saveNamePrefix 'Event'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'testClips','C');
end
fprintf('done!\n\n');
clear testClips

NORMALIZATION = [1 8.540838e-001 1.451333e+001 4.480513e+000 3.008648e-002 4.305218e+002 1.336457e-001 1.157824e+002 3.351075e-001 6.731011e-003 2.713941e+004];

%%%%% Location Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load([loadNamePrefix 'Location']);

instances = findLabels(testClips, 1)
fprintf('################################### Location ###################################\n');
fprintf('Computing features for test clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    audio = double(testClips{i}.audio)/2^15;
    numFrames = floor( (length(audio) - (C.Location.FRAMESIZE-C.Location.FRAMESTEP)) / C.Location.FRAMESTEP );
    featureTimestamps = linspace(testClips{i}.audioStart,testClips{i}.audioStop,numFrames)';
    features_temp = speech_features(audio,C.Location.FRAMESIZE,C.Location.FRAMESTEP,C.Location.rms/5)';
    testClips{i}.audioFeatures = [featureTimestamps features_temp(:,[4 6 3 7 1])]; % same order of features as in c-code
    testClips{i}.audioFeaturesLocation = AccelFeature(testClips{i},'audio_mean_var',C.Location.WINDOWSIZE,C.Location.TIMESTEP,C.Location.NUM_FEATURES);
    testClips{i}.audioFeaturesLocation = testClips{i}.audioFeaturesLocation ./ (ones(size(testClips{i}.audioFeaturesLocation,1),1)*NORMALIZATION);
    testClips{i}.audioFeatures = [];  % free memory
end
fprintf('\n'); clear audio clip;

fprintf('Classifying clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.classificationLocation = [GClassifier(testClips{i}.audioFeaturesLocation,C.Location.bnet)...
        ones(size(testClips{i}.audioFeaturesLocation,1),1)*(testClips{i}.label(C.Location.INTERESTING_LABEL_IDX))];
end
fprintf('\n');
[C.Location.accuracy C.Location.confusion] = EvaluateClassifications(testClips,'location',C.Location.LABEL_STRINGS);
if (saveResults)
    saveName = [saveNamePrefix 'Location'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'testClips','C');
end
fprintf('done!\n\n');
if (saveResults)
    saveName = [saveNamePrefix 'C'];
    fprintf('Saving classifers as %s.mat...',saveName);
    save(saveName,'C');
end
fprintf('done!\n');
clear testClips

%%%%% Speech Classification %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load([CloadNamePrefix 'Speech']);
load([loadNamePrefix 'Speech']);

instances = findLabels(testClips, 2)
fprintf('################################### Speech ###################################\n');

fprintf('Computing features for test clips');
for i=1:length(testClips) 
    fprintf(' %d',i);
    audio = double(testClips{i}.audio)/2^15;
    numFrames = floor( (length(audio) - (C.Speech.FRAMESIZE-C.Speech.FRAMESTEP)) / C.Speech.FRAMESTEP );
    featureTimestamps = linspace(testClips{i}.audioStart,testClips{i}.audioStop,numFrames)';
    features_temp = speech_features(audio,C.Speech.FRAMESIZE,C.Speech.FRAMESTEP,C.Speech.rms/5)';
    testClips{i}.audioFeatures = [featureTimestamps features_temp(:,[4 6 3 7 1])]; % same order of features as in c-code
    testClips{i}.audioFeaturesSpeech = AccelFeature(testClips{i},'audio_mean_var',C.Speech.WINDOWSIZE,C.Speech.TIMESTEP,C.Speech.NUM_FEATURES);
    testClips{i}.audioFeaturesSpeech = testClips{i}.audioFeaturesSpeech ./ (ones(size(testClips{i}.audioFeaturesSpeech,1),1)*NORMALIZATION);
    testClips{i}.audioFeatures = [];  % free memory
end
fprintf('\n'); clear audio clip;

fprintf('Classifying clips');
for i=1:length(testClips)
    fprintf(' %d',i);
    testClips{i}.classificationSpeech = [GClassifier(testClips{i}.audioFeaturesSpeech,C.Speech.bnet)...
        ones(size(testClips{i}.audioFeaturesSpeech,1),1)*(testClips{i}.label(C.Speech.INTERESTING_LABEL_IDX))];
end
fprintf('\n');
[C.Speech.accuracy C.Speech.confusion] = EvaluateClassifications(testClips,'speech',C.Speech.LABEL_STRINGS);
speakingConfusion = [C.Speech.confusion(1,1) sum(C.Speech.confusion(1,2:end)); sum(C.Speech.confusion(2:end,1)) sum(sum(C.Speech.confusion(2:end,2:end)))]
if (saveResults)
    saveName = [saveNamePrefix 'Speech'];
    fprintf('Saving classified training clips as %s.mat...',saveName);
    save(saveName,'testClips','C');
end
fprintf('done!\n\n');
%clear testClips

totalRunTime = etime(clock,startTime);
fprintf('Total runtime was %5.0f seconds = %2.2f hours\n',totalRunTime,totalRunTime/60/60);