%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Written by Mark Blum (mark@media.mit.edu) in September 2005
%%%
%%% Cuts up previously loaded training data into training and testing
%%% cases. HMMs are trained with 2, 3 and 4 states per chain.
%%% A normalization scheme is used to make loglikelihoods comparable
%%% 

close all; clear all; drawnow;
startTime = clock;
saveResults = 1;

DRIVE_LETTER = 'E';
loadNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set2\set2_');
saveNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set3\set3_HMM_SPC12_');
load([loadNamePrefix 'Event']);
[a b c] = findLabels(trainingClips,4,8,0); % leave away the manipulation class
trainingClips = trainingClips(c);

numClips = size(trainingClips,2);
%load('E:\DiaryAppData\000_workspace\set3\set3_HMM_SPC12_allCases_final')

DEF_C.name = 'HMM_event_finnal_SPC12';
DEF_C.INTERESTING_LABEL_IDX = 5;   % with the current labelling categories, this means 'Event'
DEF_C.MAXITER=100;                  % The number of iterations of EM (max)
DEF_C.EPSILON=1e-20;               % Stopping criterion for the EM (not used)
DEF_C.WINDOWSIZE = 32; %10         % [samples] features will be computed over this window
DEF_C.SPC = 12; %40                % [slices] slices per classification case
DEF_C.TIMESTEP = DEF_C.WINDOWSIZE/87.3;  % [seconds] time length of a slice, 87.3 is the sampling rate
DEF_C.LABEL_STRINGS = {'no event','eating','typing','shaking hands','clapping hands','driving','brushing teeth','doing the dishes'};
DEF_C.NUM_CLASSES = length(DEF_C.LABEL_STRINGS);
DEF_C.NUM_STATES = 1;              % number of states per HMM
DEF_C.featureType = 'mean_var';
DEF_C.NUM_FEATURES = 12;
DEF_C.NORMALIZE = 0;               % legacy, an automatic normalization scheme was tested, but not really successfully
DEF_C.HMM_normalization = [ones(DEF_C.NUM_CLASSES,1) zeros(DEF_C.NUM_CLASSES,1)];  %this corresponds to no normalization
DEF_C.UNKNOWN_BONUS = 0;           % legacy, special case eqivalent to classBonuses = [UNKNOWN_BONUS 0 0 0 0 0 ...]
DEF_C.classBonuses = zeros(1,DEF_C.NUM_CLASSES);  % Bonus logliks that will be added like priors. They will be computed below
RANDSEED = 123456; %it seems that the result of EM-learning can depend quite a bit on its initialization

kk = 1;
C{kk} = DEF_C;

%%% Compute features of length WINDOWSIZE. These features will need to be
%%% gouped in chunks of size SPC!

fprintf('Computing features for %d clips ',numClips);
testCases = cell(1,C{kk}.NUM_CLASSES);
for i=1:numClips
    fprintf('%d ',i);
    if (mod(i,20)==0) fprintf('\n '); end
    trainingClips{i}.features = AccFeatures(trainingClips{i},C{kk}.featureType,C{kk}.WINDOWSIZE,C{kk}.TIMESTEP,C{kk}.NUM_FEATURES);
    label = trainingClips{i}.label(C{kk}.INTERESTING_LABEL_IDX)+1;
    testCases{label} = [testCases{label}; trainingClips{i}.features(1:floor(size(trainingClips{i}.features,1)/C{kk}.SPC)*C{kk}.SPC,:)];
end
fprintf('\n');
save([saveNamePrefix 'allCases_final'],'testCases','DEF_C');

%%% Create a 50 / 50 train and testset by interleaving cases
%%% this is a bit complicated but necessary since the slices have
%%% WINDOWSIZE samples and each classification case has SPC slices per case

maxNumTrainingCases = length(testCases{1});  %1000*C{kk}.SPC;
leaveOneOutMask = [ ones(DEF_C.SPC,ceil(maxNumTrainingCases/DEF_C.SPC)) ; zeros(DEF_C.SPC,ceil(maxNumTrainingCases/DEF_C.SPC)) ];
for i=1:C{kk}.NUM_CLASSES, 
    %numTrainingCases = size(testCases{i},2); 
    testCases{i} = num2cell(testCases{i}',1);
    %trainingCases{i} = testCases{i}(1:min([maxNumTrainingCases length(testCases{i})])); 
    %trainingCases{i} = testCases{i}(1:min([15000 length(testCases{i})])); 
    trainIdx = find(leaveOneOutMask(1:length(testCases{i})) == 1);
    testIdx = find(leaveOneOutMask(1:length(testCases{i})) == 0);
    trainingCases{i} = testCases{i}(trainIdx); 
    testCases{i} = testCases{i}(testIdx); 
end

fprintf('Number of trainingCases: ');
trainingCases
fprintf('Number of testCases: ');
testCases

%%%%% kk = 1, 2 states %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

kk = 1;
C{kk} = DEF_C;
C{kk}.NUM_STATES = 2;

rand('state',kk*RANDSEED);
for i=1:C{kk}.NUM_CLASSES
    C{kk}.bnet(i) = mk_influence(C{kk}.NUM_STATES, C{kk}.NUM_FEATURES,'c');
    C{kk}.engine(i) = influence_inf_engine(C{kk}.bnet(i));
end

figure;
for i=1:C{kk}.NUM_CLASSES
    fprintf('Training HMM for class %d\n',i);
    subplot(floor((C{kk}.NUM_CLASSES+1)/2),2,i); %Note: if Wen's dll is used, then no transition matricies are plotted
    if i==1, 
        C{kk}.MAXITER = floor(DEF_C.MAXITER*0.7); % to speed up a little
    else 
        C{kk}.MAXITER = DEF_C.MAXITER;
    end
    [C{kk}.bnet_trained(i)] = learn_params_influence(C{kk}.engine(i),trainingCases{i},C{kk}.MAXITER);  %Note: no LL_trace is returned with the dll
    C{kk}.engine_trained(i) = influence_inf_engine(C{kk}.bnet_trained(i));
end
  
[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(trainingCases, C{kk}, 0);
kk = kk+1;
C{kk}=C{kk-1};
[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(testCases, C{kk}, 0); 
kk = kk+1;
C{kk}=C{kk-1};
C{kk}.classBonuses = zeros(1,C{kk}.NUM_CLASSES);


%%% The following lines of code have the purpose to compute loglik bonus
%%% values for each class. This is somewhat equivalent to assigning priors
%%% The goal is to "normalize" all HMMs to obtain fairness, and thus higher
%%% accuracies.

%%% to acheive a good balance bewteen overall and average class accuracy
%%% we chose the following weights for a more meaningful accuracy measure
C{kk}.relativeCaseCounts = [0.3 0.2 0.2 0.05 0.05 0.075 0.06125 0.06125];

%%% this approach here uses the number of cases as weights. Classes with 
%%% few samples were not weighted enough
% for i=1:C{kk}.NUM_CLASSES
%     C{kk}.relativeCaseCounts(i) = length(testCases{i});
% end
% C{kk}.relativeCaseCounts(1) = C{kk}.relativeCaseCounts(1)/2;
% C{kk}.relativeCaseCounts = normalize(C{kk}.relativeCaseCounts)

%%% alogrithm description: add and subtract K to the loglik of each class
%%% Compare classification accuracies for each combination (numClasses ^ 2)
%%% Select the best, and use the corresponding K values as classBonuses
%%% Make K smaller and start over.
%%% It is obvious that this is neither fast, nor tremendously exact
%%% but it is fast enough and exact enough :-)

for i=1:12
    bestClassBonuses = C{kk}.classBonuses;
    K = 61-i*5;
    B(:,1) = bestClassBonuses'+K; B(:,2) = bestClassBonuses'-K;
    count = 1; maxAcc = 0;
    for b1=B(1,:), for b2=B(2,:), for b3=B(3,:), for b4=B(4,:), for b5=B(5,:), for b6=B(6,:), for b7=B(7,:), for b8=B(8,:)
                                    if mod(count,100)==0
                                        fprintf('%d ',count);
                                    end
                                    C{kk}.classBonuses = [b1 b2 b3 b4 b5 b6 b7 b8];
                                    [acc confusion] = classifyHMM_influence(trainingCases, C{kk}, -1, C{kk-2}.loglike);
                                    averageAcc = sum(100*(diag(confusion)./sum(confusion,2))'.*C{kk}.relativeCaseCounts);
                                    if (averageAcc > maxAcc)
                                        bestClassBonuses = C{kk}.classBonuses;
                                        fprintf('%2.0f%% ',averageAcc);
                                        maxAcc = averageAcc; 
                                    end
                                    count = count + 1;
    end, end, end, end, end, end, end, end
    fprintf('\n');
    C{kk}.classBonuses = bestClassBonuses;
end
%%% Evaluate the trainingCases based on the new class bonuses
[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(trainingCases, C{kk}, 0, C{kk-2}.loglike);
averageAcc = sum(100*(diag(C{kk}.confusion)./sum(C{kk}.confusion,2))'.*C{kk}.relativeCaseCounts)

%%% Evaluate the testCases based on the same class bonuses
kk = kk+1;
C{kk}=C{kk-1};
C{kk}.classBonuses = C{kk-1}.classBonuses;
[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(testCases, C{kk}, 0, C{kk-2}.loglike);
averageAcc = sum(100*(diag(C{kk}.confusion)./sum(C{kk}.confusion,2))'.*C{kk}.relativeCaseCounts)


%%%%% 3 States %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

kk = 6;
C{kk} = DEF_C;
C{kk}.NUM_STATES = 3;

rand('state',kk*RANDSEED);
for i=1:C{kk}.NUM_CLASSES
    C{kk}.bnet(i) = mk_influence(C{kk}.NUM_STATES, C{kk}.NUM_FEATURES,'c');
    C{kk}.engine(i) = influence_inf_engine(C{kk}.bnet(i));
end

figure;
for i=1:C{kk}.NUM_CLASSES
    fprintf('Training HMM for class %d\n',i);
    subplot(floor((C{kk}.NUM_CLASSES+1)/2),2,i);
     if i==1, 
        C{kk}.MAXITER = floor(DEF_C.MAXITER*0.7); 
    else 
        C{kk}.MAXITER = DEF_C.MAXITER;
    end
    [C{kk}.bnet_trained(i) ] = learn_params_influence(C{kk}.engine(i),trainingCases{i},C{kk}.MAXITER);
    C{kk}.engine_trained(i) = influence_inf_engine(C{kk}.bnet_trained(i));
end

[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(trainingCases, C{kk}, 0);
kk = kk+1;
C{kk}=C{kk-1};
[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(testCases, C{kk}, 0); 
kk = kk+1;
C{kk}=C{kk-2};
C{kk}.classBonuses = zeros(1,C{kk}.NUM_CLASSES);
C{kk}.relativeCaseCounts = [0.3 0.2 0.2 0.05 0.05 0.075 0.06125 0.06125];
for i=1:12
    bestClassBonuses = C{kk}.classBonuses;
    K = 61-i*5;
    B(:,1) = bestClassBonuses'+K; B(:,2) = bestClassBonuses'-K;
    count = 1; maxAcc = 0;
    for b1=B(1,:), for b2=B(2,:), for b3=B(3,:), for b4=B(4,:), for b5=B(5,:), for b6=B(6,:), for b7=B(7,:), for b8=B(8,:)
                                    if mod(count,100)==0
                                        fprintf('%d ',count);
                                    end
                                    C{kk}.classBonuses = [b1 b2 b3 b4 b5 b6 b7 b8];
                                    [acc confusion] = classifyHMM_influence(trainingCases, C{kk}, -1, C{kk-2}.loglike);
                                    averageAcc = sum(100*(diag(confusion)./sum(confusion,2))'.*C{kk}.relativeCaseCounts);
                                    if (averageAcc > maxAcc)
                                        bestClassBonuses = C{kk}.classBonuses;
                                        fprintf('%2.0f%% ',averageAcc);
                                        maxAcc = averageAcc; 
                                    end
                                    count = count + 1;
    end, end, end, end, end, end, end, end
    fprintf('\n');
    C{kk}.classBonuses = bestClassBonuses;
end
[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(trainingCases, C{kk}, 0, C{kk-2}.loglike);
averageAcc = sum(100*(diag(C{kk}.confusion)./sum(C{kk}.confusion,2))'.*C{kk}.relativeCaseCounts)

kk = kk+1;
C{kk}=C{kk-1};
C{kk}.classBonuses = C{kk-1}.classBonuses;
[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(testCases, C{kk}, 0, C{kk-2}.loglike);
averageAcc = sum(100*(diag(C{kk}.confusion)./sum(C{kk}.confusion,2))'.*C{kk}.relativeCaseCounts)


%%%%% 4 states %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

kk = 11;
C{kk} = DEF_C;
C{kk}.NUM_STATES = 4;
%C{kk}.MAXITER = 40;

rand('state',kk*RANDSEED);
for i=1:C{kk}.NUM_CLASSES
    C{kk}.bnet(i) = mk_influence(C{kk}.NUM_STATES, C{kk}.NUM_FEATURES,'c');
    C{kk}.engine(i) = influence_inf_engine(C{kk}.bnet(i));
end

figure;
for i=1:C{kk}.NUM_CLASSES
    fprintf('Training HMM for class %d\n',i);
    subplot(floor((C{kk}.NUM_CLASSES+1)/2),2,i);
     if i==1, 
        C{kk}.MAXITER = floor(DEF_C.MAXITER*0.7); 
    else 
        C{kk}.MAXITER = DEF_C.MAXITER;
    end
    [C{kk}.bnet_trained(i) ] = learn_params_influence(C{kk}.engine(i),trainingCases{i},C{kk}.MAXITER);
    C{kk}.engine_trained(i) = influence_inf_engine(C{kk}.bnet_trained(i));
end

[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(trainingCases, C{kk}, 0);
kk = kk+1;
C{kk}=C{kk-1};
[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(testCases, C{kk}, 0); 
kk = kk+1;
C{kk}=C{kk-2};
C{kk}.classBonuses = zeros(1,C{kk}.NUM_CLASSES);
C{kk}.relativeCaseCounts = [0.3 0.2 0.2 0.05 0.05 0.075 0.06125 0.06125];
for i=1:12
    bestClassBonuses = C{kk}.classBonuses;
    K = 61-i*5;
    B(:,1) = bestClassBonuses'+K; B(:,2) = bestClassBonuses'-K;
    count = 1; maxAcc = 0;
    for b1=B(1,:), for b2=B(2,:), for b3=B(3,:), for b4=B(4,:), for b5=B(5,:), for b6=B(6,:), for b7=B(7,:), for b8=B(8,:)
                                    if mod(count,100)==0
                                        fprintf('%d ',count);
                                    end
                                    C{kk}.classBonuses = [b1 b2 b3 b4 b5 b6 b7 b8];
                                    [acc confusion] = classifyHMM_influence(trainingCases, C{kk}, -1, C{kk-2}.loglike);
                                    averageAcc = sum(100*(diag(confusion)./sum(confusion,2))'.*C{kk}.relativeCaseCounts);
                                    %averageAcc = acc;
                                    if (averageAcc > maxAcc)
                                        bestClassBonuses = C{kk}.classBonuses;
                                        fprintf('%2.0f%% ',averageAcc);
                                        maxAcc = averageAcc; 
                                    end
                                    count = count + 1;
    end, end, end, end, end, end, end, end
    fprintf('\n');
    C{kk}.classBonuses = bestClassBonuses;
end
[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(trainingCases, C{kk}, 0, C{kk-2}.loglike);
averageAcc = sum(100*(diag(C{kk}.confusion)./sum(C{kk}.confusion,2))'.*C{kk}.relativeCaseCounts)

kk = kk+1;
C{kk}=C{kk-1};
C{kk}.classBonuses = C{kk-1}.classBonuses;
[C{kk}.acc  C{kk}.confusion C{kk}.t_classify C{kk}.decision C{kk}.loglike] = classifyHMM_influence(testCases, C{kk}, 0, C{kk-2}.loglike);
averageAcc = sum(100*(diag(C{kk}.confusion)./sum(C{kk}.confusion,2))'.*C{kk}.relativeCaseCounts)
save([saveNamePrefix 'C'],'C');
return


%%% GMM as a reference
% D = DEF_C;
% D.MAXITER=3;                   % The number of iterations of EM (max)
% D.EPSILON=1e-1000;             % Stopping criterion for the EM
% D.WINDOWSIZE = 3*128;             % [samples] features will be computed over this window
% D.TIMESTEP = 2;                % [seconds] classification is done once every TIMESTEP seconds
% D.featureType = 'mean_var';
% D.NUM_FEATURES = 12;
% [testX testLabels] = computeFeatures(trainingClips, D);
% trainingX = testX(1:1:length(testX),:);
% trainingLabels = testLabels(1:1:length(testX),:);
% computeLDA(trainingX, trainingLabels, 3);
% legend(D.LABEL_STRINGS);  
% [D.bnet D.t_learn]   = trainBNET(trainingX, trainingLabels, D);
% priors = get_field(D.bnet.CPD{1},'cpt');
% priors(1) = priors(1)*1.3;
% priors = normalize(priors);
% D.bnet.CPD{1} = tabular_CPD(D.bnet,1,'CPT',priors);
% [D.acc  D.confusion D.t_classify] = classifyBNET(testX, testLabels, D); 


%%% plot learing curves. Here disabled because no LL_trace is returned by the dll
% LL_evaluation = [];
% colors = {'b', 'g', 'r', 'c', 'm', 'y', 'k', 'b-', 'g-', 'r-', 'c-', 'm-', 'y-', 'k-','b.', 'g.', 'r.', 'c.', 'm.', 'y.', 'k.'};
% for j=1:DEF_C.NUM_CLASSES
%     figure; hold on;
%     for i=1:length(C)
%         LL_evaluation(i,j) = C{i}.LLtrace{j}(end);
%         plot(C{i}.LLtrace{j}(2:end),colors{i});
%     end
%     hold off;
% end
% LL_evaluation


%%% legacy, Select the best training results from all C{kk}s and create C{final}
% final = kk+1;
% C{final} = DEF_C;
% for j=1:C{final}.NUM_CLASSES
%     [C{final}.LL_final(j) idx] = max(LL_evaluation(:,j));
%     C{final}.bnet_trained(j) = C{idx}.bnet_trained(j);
%     C{final}.engine_trained(j) = C{idx}.engine_trained(j);
% end
% [C{final}.acc  C{final}.confusion C{final}.t_classify] = classifyHMM_influence(testCases, C{final});

%%%%% Summarize results %%%%%%%%%%%%%%%%%%%%%%

% for i=1:length(C)
%     display(sprintf('C %2.0f,\tWIN %4.0f,\tN_FEAT %2.0f,\tSTATES %2.0f,\tNRM %d,\ttClass %5.0f,\tacc %3.1f%%, \t''%s''',...
%         i,C{i}.WINDOWSIZE,C{i}.NUM_FEATURES,C{i}.NUM_STATES,C{i}.NORMALIZE,C{i}.t_classify,C{i}.acc,C{i}.featureType));
% end

%%%%% Save Classifiers and all figures %%%%%%%%%%%%%%%%%%%%%%
if (saveResults)
    saveName = [saveNamePrefix 'C'];
    fprintf('saving classifiers as %s...',saveName);
    save(saveName,'C');
%     for i=1:(kk)*3
%         try
%             %if (i<10), c = '0'; else, c = ''; end
%             hgsave(i,sprintf('%s_%d',DEF_C.name,i));
%         catch
%             display(sprintf('Warning: could not save figure %d',i));
%         end
%     end
end
fprintf('done!\n');
totalRunTime = etime(clock,startTime);
display(sprintf('total runtime was %5.0f seconds = %2.2f hours',totalRunTime,totalRunTime/60/60));



%%% This code will create a plot with all features, colored corresponding
%%% to their equivalent state in the HMM
CLASS = 6;
CLASSIFIER = 14;
TEXT_LABEL_INDEX = 4;  % this will print out the posutre corresponding to the Clips
REF_LABEL_STRINGS = {'unknown','lie','sit','stand','walk','run','bike'};
mpe = influence_mpe(C{CLASSIFIER}.engine_trained(CLASS),testCases{CLASS});
abc = cell2num(testCases{CLASS});
X_ZOOM = 1;
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
%selectedFeaturesIdx =1:C{CLASSIFIER}.NUM_FEATURES;
selectedFeaturesIdx = [4 5 6 10 11 12];
for i=1:length(selectedFeaturesIdx)
    subplot(length(selectedFeaturesIdx)+1,1,i)
    plot(find(mpe==1),abc(selectedFeaturesIdx(i),find(mpe==1)),'b.')
    hold on;
    plot(find(mpe==2),abc(selectedFeaturesIdx(i),find(mpe==2)),'g.')
    plot(find(mpe==3),abc(selectedFeaturesIdx(i),find(mpe==3)),'r.')
    plot(find(mpe==4),abc(selectedFeaturesIdx(i),find(mpe==4)),'m.')
    %plot(find(mpe==5),abc(selectedFeaturesIdx(i),find(mpe==5)),'k.')
    %plot(find(mpe==6),abc(selectedFeaturesIdx(i),find(mpe==6)),'c.')
    axis tight; a = axis; axis([a(1) a(2)/X_ZOOM a(3) a(4)]);
end
subplot(C{CLASSIFIER}.NUM_FEATURES+1,1,C{CLASSIFIER}.NUM_FEATURES+1)
plot((C{CLASSIFIER}.SPC:C{CLASSIFIER}.SPC:(length(C{CLASSIFIER}.decision{CLASS})*C{CLASSIFIER}.SPC)+C{CLASSIFIER}.SPC-1),C{CLASSIFIER}.decision{CLASS},'b.')
axis tight; a = axis; axis([a(1) a(2)/X_ZOOM 1 C{CLASSIFIER}.NUM_CLASSES]); title(sprintf('Ground truth: %s',C{CLASSIFIER}.LABEL_STRINGS{CLASS}));

%%%%% Print the label of context dimension TEXT_LABEL_INDEX into the plots
%%%%% this should help visualize the current context of other dimensions
labelPositions = [0];
labelStrings = {};
for i=1:length(trainingClips)
    if (trainingClips{i}.label(C{CLASSIFIER}.INTERESTING_LABEL_IDX)==CLASS-1)    % labels in the BNET are offset by 1 (index zero not allowed)
        labelPositions = [labelPositions labelPositions(end)+length(find(caseParentClip{CLASS}==i))*C{CLASSIFIER}.SPC];
        labelStrings = [labelStrings REF_LABEL_STRINGS(trainingClips{i}.label(TEXT_LABEL_INDEX)+1)];
        line([labelPositions(end-1) labelPositions(end-1)],[1 C{CLASSIFIER}.NUM_CLASSES],'color',[0.8 0.8 0.8]);
    end; 
end
labelPositions = labelPositions(1:end-1); labelPositionsY = mod([1:length(labelPositions)],7)/12*C{CLASSIFIER}.NUM_CLASSES+1.2;
H = text(labelPositions, labelPositionsY, labelStrings);  set(H,'FontSize',8);

% fprintf('values for eating (class 2)\n');
% for i=1:length(C)
%     get_field(C{i}.bnet_trained(2).CPD{2},'mean')
% end
% for i=1:length(C)
%     get_field(C{i}.bnet_trained(2).CPD{1},'cpt')
% end
% for i=1:length(C)
%     get_field(C{i}.bnet_trained(2).CPD{3},'cpt')
% end
% 
% fprintf('values for typing (class 3)\n');
% for i=1:length(C)
%     get_field(C{i}.bnet_trained(3).CPD{2},'mean')
% end
% for i=1:length(C)
%     get_field(C{i}.bnet_trained(3).CPD{1},'cpt')
% end
% for i=1:length(C)
%     get_field(C{i}.bnet_trained(3).CPD{3},'cpt')
% end


