%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Written by Mark Blum (mark@media.mit.edu) in September 2005
%%%
%%% This script will load all files generated in a SignalRecorder
%%% session and verify the features and classifications computed
%%% in real-time against the Matlab computations (gold).
%%% 

format compact;
close all; drawnow;
DRIVE_LETTER = 'E';
loadNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\2g_MITes\verify2\verify2_z6000_12_57_34');
saveNamePrefix = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set2\verification\');
classifiers_set1 = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set1\set1_preclassified_C');
classifier_Speech = strcat(DRIVE_LETTER,':\DiaryAppData\000_workspace\set2\Speech_C');

fprintf('######################### loading c45 classifications into testClips #########################\n');

load(classifiers_set1);

clip = SignalLoader(loadNamePrefix,'no clips',1,'no audio',0,'clean data',0,'normalize','f','plots',0,...
    'oneG_correction',0,'audio samples per line',1024,'no acc',0,'preserve timestamps',1);
clip = clip{1};
plotClips({clip},'acc1');
fprintf('Loading feature-data...');
accFeat = load([loadNamePrefix '.accelfeat']);
audioFeat = load([loadNamePrefix '.audiofeat']);
fprintf('done\nLoading classifications...');
postureClass = load([loadNamePrefix '.posture.class']);
eventClass = load([loadNamePrefix '.event.class']);
locationClass = load([loadNamePrefix '.location.class']);
speechClass = load([loadNamePrefix '.speech.class']);
fprintf('done\n');

% shift timestamps to nicer values
t0 = min([clip.acc1(1,1) clip.acc2(1,1)]);
clip.acc1(:,1) = clip.acc1(:,1) - t0;
clip.acc2(:,1) = clip.acc2(:,1) - t0;
clip.audioStart = clip.audioStart - t0;
clip.audioStop = clip.audioStop - t0;
accFeat(:,1) = accFeat(:,1) - t0;
audioFeat(:,1) = audioFeat(:,1) - t0;
postureClass(:,1) = postureClass(:,1) - t0;
eventClass(:,1) = eventClass(:,1) - t0;
locationClass(:,1) = locationClass(:,1) - t0;
speechClass(:,1) = speechClass(:,1) - t0;

%add maximum likelyhood decisions
[foo decision] = max(postureClass(:,2:end),[],2);
postureClass = [postureClass decision];
[foo decision] = max(eventClass(:,2:end),[],2);
eventClass = [eventClass decision];
[foo decision] = max(locationClass(:,2:end),[],2);
locationClass = [locationClass decision];
[foo decision] = max(speechClass(:,2:end),[],2);
speechClass = [speechClass decision];

drawnow;
firstTimeStamp = accFeat(1,1);
fprintf('Computing acceleration features...');
accFeatGold = AccelFeature(clip,C.Event.featureType,C.Event.WINDOWSIZE,C.Event.TIMESTEP,C.Event.NUM_FEATURES,firstTimeStamp);
fprintf('done\n');

fprintf('Computing audio features...');
NORMALIZATION = [1 8.540838e-001 1.451333e+001 4.480513e+000 3.008648e-002 4.305218e+002 1.336457e-001 1.157824e+002 3.351075e-001 6.731011e-003 2.713941e+004];
START = 60*10; STOP = 60*20;
audio = double(clip.audio(8000*START:8000*STOP))/2^15;
numFrames = floor( (length(audio) - (C.Location.FRAMESIZE-C.Location.FRAMESTEP)) / C.Location.FRAMESTEP );
featureTimestamps = linspace(clip.audioStart+START,clip.audioStart+STOP,numFrames)';
features_temp = speech_features(audio,C.Location.FRAMESIZE,C.Location.FRAMESTEP,C.Location.rms/5)';
clip.audioFeatures = [featureTimestamps features_temp(:,[4 6 3 7 1])]; % same order of features as in c-code
audioFeatGold = AccelFeature(clip,'audio_mean_var',C.Location.WINDOWSIZE,C.Location.TIMESTEP,C.Location.NUM_FEATURES);
audioFeatGold = audioFeatGold ./ (ones(size(audioFeatGold,1),1)*NORMALIZATION);
fprintf('done\n');

fprintf('Computing classificatins...');
postureClassGold = GClassifier(accFeatGold,C.Posture.bnet);
eventClassGold = GClassifier(accFeatGold,C.Event.bnet);
%locationClassGold = GClassifier(audioFeatGold,C.Location.bnet);
locationClassGold = GClassifier(audioFeat,C.Location.bnet);
load(classifier_Speech); % Speech used set2 C.Speech because laughing was included
%speechClassGold = GClassifier(audioFeatGold,C.Speech.bnet);
speechClassGold = GClassifier(audioFeat,C.Speech.bnet);
fprintf('done\n');
drawnow;

% plot acceleration Features
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
subplot(4,1,1);
plot(accFeat(:,1),accFeat(:,2),'b',accFeatGold(:,1),accFeatGold(:,2),'g');
title('accFeat(:,2) mean(X1)'); legend('Reatime','Matlab');
subplot(4,1,2);
plot(accFeat(:,1),accFeat(:,3),'b',accFeatGold(:,1),accFeatGold(:,3),'g');
title('accFeat(:,3) mean(Y1)'); legend('Reatime','Matlab');
subplot(4,1,3);
plot(accFeat(:,1),accFeat(:,12),'b',accFeatGold(:,1),accFeatGold(:,12),'g');
title('accFeat(:,12) Var(Y2)'); legend('Reatime','Matlab');
subplot(4,1,4);
plot(accFeat(:,1),accFeat(:,13),'b',accFeatGold(:,1),accFeatGold(:,13),'g');
title('accFeat(:,13) Var(Y3)'); legend('Reatime','Matlab');


bigfeatureStrings = {'maxPeak-Mean','numPeak-Mean','Spec Entropy-Mean','Energy-Mean','formant frequency-Mean',...
    'maxPeak-Variance','numPeak-Variance','Spec Entropy-Variance','Energy-Variance','formant frequency-Variance'};
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
for i=1:5
    subplot(5,1,i);
    plot(audioFeat(:,1),audioFeat(:,1+i),'b',audioFeatGold(:,1),audioFeatGold(:,1+i),'g');
    title(bigfeatureStrings{i});
end
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
for i=6:10
    subplot(5,1,i-5);
    plot(audioFeat(:,1),audioFeat(:,1+i),'b',audioFeatGold(:,1),audioFeatGold(:,1+i),'g');
    title(bigfeatureStrings{i});
end


% plot class probabilities
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
subplot(2,1,1);
plot(postureClass(:,1),postureClass(:,4),'b',postureClassGold(:,1),postureClassGold(:,4),'g');
title('postureClass(:,4) sitting'); legend('Reatime','Matlab');
subplot(2,1,2);
plot(postureClass(:,1),postureClass(:,5),'b',postureClassGold(:,1),postureClassGold(:,5),'g');
title('postureClass(:,5) walking'); legend('Reatime','Matlab');

set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
subplot(2,1,1);
plot(locationClass(:,1),locationClass(:,4),'b',locationClassGold(:,1),locationClassGold(:,4),'g');
title('locationClass(:,4) sitting'); legend('Reatime','Matlab');
subplot(2,1,2);
plot(locationClass(:,1),locationClass(:,5),'b',locationClassGold(:,1),locationClassGold(:,5),'g');
title('locationClass(:,5) walking'); legend('Reatime','Matlab');


% plot hard decisions
set(figure,'Color',[1 1 1],'Position',[5 29 1270 920]);
subplot(4,1,1);
plot(postureClass(:,1),postureClass(:,end),'b',postureClassGold(:,1),postureClassGold(:,end)+0.02,'g');
title('postureClass(:,end) decisions'); legend('Reatime','Matlab');
subplot(4,1,2);
plot(eventClass(:,1),eventClass(:,end),'b',eventClassGold(:,1),eventClassGold(:,end)+0.02,'g');
title('eventClass(:,end) decisions'); legend('Reatime','Matlab');
subplot(4,1,3);
plot(locationClass(:,1),locationClass(:,end),'b',locationClassGold(:,1),locationClassGold(:,end)+0.02,'g');
title('locationClass(:,end) decisions'); legend('Reatime','Matlab');
subplot(4,1,4);
plot(speechClass(:,1),speechClass(:,end),'b',speechClassGold(:,1),speechClassGold(:,end)+0.02,'g');
title('speechClass(:,end) decisions'); legend('Reatime','Matlab');

